/*
 * DAOerfassenSchueler.cs
 */
using System;
using System.IO;

using Beringer.oopBuch.ModelKlassen;

namespace Beringer.oopBuch.DaoKlassen
{
    /**
    <summary>
    DAO-Klasse für das Erfassen von Schülerdaten von der 
    Standardeingabe (= Tastatur).
    <p>
    Die Schülerdaten werden in 'Schueler'-Objekten aufbereitet.
    </p>
    <p>
    Methoden 'oeffnen' und 'schliessen' sind nicht notwendig, weil
    Console.In (= Standardeingabe Tastatur) standardmäßig immer geöffnet ist.
    </p>

    <p>
    Seit: November 2004 <br />
    Änderung: Juni 2007 (Refaktorierung) <br />
    Änderung: September 2012 (Refaktorierung - lesenEingabe) <br />
    Änderung: August 2014 (neue Namensbereiche 'ModelKlassen' und 'DaoKlassen')
    </p>

    <seealso cref="Schueler" />
    <seealso cref="DAOschreibenSchueler" />
    <seealso cref="DAOlesenSchueler" />
    <seealso cref="Beringer.oopBuch.EinAusgabe.ErfassenSchreiben" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    <ul>
      <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
      <li>'^Z' (= 'Strg' + 'Z') beendet die Eingabe (EOF) und liefert 'null' für
          die Eingabe zurück.</li>
      <li>Umgekehrt wird 'null' nur bei '^Z' als Eingabe zurückgegeben.</li>
      <li>EOF ist also äquivalent zur Eingabe von '^Z'.</li>
      <li>Achtung: 'EndOfStreamException' tritt bei der Standardeingabe nicht 
	               ein.</li>
      <li>Zusätzlich wird auch die Eingabe der Zeichenfolge "EOF" als EOF 
          interpretiert (gleichbedeutend mit der Eingabe von '^Z').</li>
      <li>Bei einem allgemeinen Eingabefehler ('IOException') wird 'null' als 
          Eingabe definiert (das ist gleichbedeutend mit EOF).</li>
    </ul>
    </remarks>
    */
    public class DAOerfassenSchueler
    {
        // ---------------------------------------------------------------------
        // Klassenkonstante
        // ---------------------------------------------------------------------

        /**
        <summary>
        Zeichenkette, die das Ende der Tastatureingabe definiert.
        </summary>
        <remarks>
        Dient als zusätzliche EOF-Kennzeichnung 
        (zusätzlich zur Eingabe von '^Z')
        </remarks>
        */
        private static readonly String  EOF = "EOF";

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        /// Datenobjekt für die zu erfassenden Daten
        private Schueler  schueler;
        private bool      eof = false;

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Get-Property für die Daten des erfassten Datensatzes.
        </summary>
        */
        public Schueler SchuelerDaten
        {
            get
            {
                return  schueler;
            }
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Get-Property für EOF (= "Ende der Datei") - nachdem alle Datensätze 
        erfasst wurden.
        <returns>
          true, wenn das Ende der Datenerfassung erreicht wurde
        </returns>
        */
        public bool Eof
        {
            get
            {
                return  eof;
            }
        }

        // ---------------------------------------------------------------------
        // Instanzmethoden für Dateiverarbeitung
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liest den nächsten Datensatz.
        <p>
        Die Datenfelder werden (entsprechend dem Satzaufbau) der Reihe nach über
        die Tastatur eingegeben (erfasst) und - soweit möglich - geprüft.<br />
        Bei einem Fehler wird kein Datensatz erstellt und die Datenerfassung 
        abgebrochen.
        </summary>

        <returns>
          true, wenn die Erfassung des Datensatzes erfolgreich war
        </returns>
        */
        public bool Lesen()
        {
            String  eingabe;

            String  name;
            float   notenschnitt;
            String  klasse;

            // --------------------
            // 1. Datenfeld (Name - Zeichenkette mit variabler Länge)
            eingabe = lesenEingabe("Bitte Schülernamen eingeben (Ende: "
                                   + EOF + " oder ^Z):");
            if (eof)
            {
                return  true;     // EOF ist kein Fehler
            }

            name = eingabe;

            // --------------------
            // 2. Datenfeld (Notendurchschnitt - Zahl)
            eingabe = lesenEingabe("Bitte Notendurchschnitt eingeben:");
            if (eof)
            {
                // Bei zu frühem EOF wird kein Datensatz erstellt
                return  true;     // EOF ist kein Fehler
            }

            try
            {
                notenschnitt = float.Parse(eingabe);
            }
            catch (FormatException fehler)
            {
                Console.WriteLine("Falscher Notendurchschnitt: " + eingabe);
                return  false;
            }

            // --------------------
            // 3. Datenfeld (Klassenbezeichnung - Zeichenkette fixer Länge)
            eingabe = lesenEingabe("Bitte Klasse eingeben (max. "
                                   + Schueler.KLASSELEER.Length + " Zeichen):");
            if (eof)
            {
                // Bei zu frühem EOF wird kein Datensatz erstellt
                return  true;     // EOF ist kein Fehler
            }

            klasse = eingabe;

            // --------------------
            // Erfasste Daten sind korrekt -> Objekt für Datensatz instanzieren

            schueler = new Schueler();
            schueler.Name = name;
            schueler.Notenschnitt = notenschnitt;
            schueler.Klasse = klasse;

            return  true;
        }

        // ---------------------------------------------------------------------
        // Private Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liest eine Zeile von der Tastatur.
        <p>
        Eingabe: alle Zeichen bis zur 'Return'- bzw. 'Enter'-Taste.
        </p>
        <p>
        Ende der Eingabe durch die Zeichenfolge "EOF" oder 
           durch '^Z' (= 'Strg' + 'Z').
        </p>
        </summary>
        <remarks>
        <ul>
          <li>'EndOfStreamException' tritt bei der Standardeingabe nicht 
              ein!</li>
          <li>'^Z' = EOF-Kennung bei Konsoleneingabe, d.h.:
              EOF ist äquivalent zur Eingabe von '^Z'.</li>
          <li>Eingabe von '^Z' liefert 'null' für die Eingabe.</li>
          <li>EOF ist also gleichbedeutend mit 'null' für die Eingabe.</li>
        </ul>
        </remarks>

        <param name="eingabeaufforderung">
          Eingabeaufforderung
        </param>

        <returns>
          eingegebene Daten oder 'null' (bei Eingabe von '^Z')
        </returns>
        */
        private String lesenEingabe(String eingabeaufforderung)
        {
            String  eingabe;

            Console.WriteLine(eingabeaufforderung);
            try
            {
                eingabe = Console.ReadLine();
            }
            catch (IOException fehler)
            {
                eingabe = null;           // Datenerfassung beenden (EOF setzen)
                Console.WriteLine("Fehler beim Erfassen: " + fehler.Message);
            }

            // EOF prüfen:
            if ( (eingabe == null) || (eingabe == EOF) )
            {
                eof = true;
                Console.WriteLine("EOF");  // diese Ausgabe dient nur zum Testen
            }

            return  eingabe;
        }
    }
}
