/*
 * DAOlesenSchueler.cs
 */
using System;
using System.IO;

using Beringer.oopBuch.ModelKlassen;

namespace Beringer.oopBuch.DaoKlassen
{
    /**
    <summary>
    DAO-Klasse für das Lesen von Schülerdaten aus einer Datei 
    (Nichtstandarddatei).
    <p>
    Die Schülerdaten werden in 'Schueler'-Objekten aufbereitet.
    </p>

    <p>
    Seit: November 2004 <br />
    Änderung: Juni 2007 (Refaktorierung) <br />
    Änderung: Juli 2012 (FileStream-Objekt als lokale Variable) <br />
    Änderung: August 2014 (neue Namensbereiche 'ModelKlassen' und 'DaoKlassen')
    </p>

    <seealso cref="Schueler" />
    <seealso cref="DAOausgebenSchueler" />
    <seealso cref="DAOerfassenSchueler" />
    <seealso cref="Beringer.oopBuch.EinAusgabe.LesenAusgeben" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    <ul>
      <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
      <li>Die Konsolausgaben bei Fehlern dienen nur zur Kontrolle und sollten 
          durch Erzeugung eigener Fehlerobjekte ersetzt werden.</li>
    </ul>
    </remarks>
    */
    public class DAOlesenSchueler
    {
        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        /// Datenobjekt für die zu lesenden Daten
        private Schueler  schueler;
        private bool      eof = false;

        /// Verbindungsobjekt zur Datei
        private BinaryReader eingabesatz;

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Get-Property für die Daten des gelesenen Datensatzes.
        </summary>
        */
        public Schueler SchuelerDaten
        {
            get
            {
                return  schueler;
            }
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Get-Property für EOF (= "Ende der Datei") - nachdem alle Datensätze 
        gelesen wurden.

        <returns>
          true, wenn das Ende der Datei erreicht wurde
        </returns>
        */
        public bool Eof
        {
            get
            {
                return  eof;
            }
        }

        // ---------------------------------------------------------------------
        // Instanzmethoden für Dateiverarbeitung
        // ---------------------------------------------------------------------

        /**
        <summary>
        Öffnet eine (Nichtstandard-)Datei zum Lesen.
        </summary>

        <param name="dateiname">
          Name der zu öffnenden Datei (als Zeichenkette)
        </param>

        <returns>
          true, wenn das Öffnen der Datei erfolgreich war
        </returns>
        */
        public bool Oeffnen(String dateiname)
        {
            FileStream  eingabedatei;               // entfällt bei Schachtelung

            try
            {
                eingabedatei = new FileStream(dateiname, FileMode.Open);
                eingabesatz  = new BinaryReader(eingabedatei);

// Schachtelung ist möglich
//              eingabesatz = new BinaryReader(
//                                    new FileStream(dateiname, FileMode.Open));
            }
            catch (IOException fehler)
            {
                Console.WriteLine("Fehler beim Öffnen der Datei: "
                                  + fehler.Message);           // nur zum Testen
                return  false;
            }

            return  true;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Liest den nächsten Datensatz aus der Datei.
        <p>
        Die Datenfelder werden (entsprechend dem Satzaufbau) der Reihe nach aus 
        der Datei gelesen.<br />
        Bei einem Fehler wird kein Datensatz erstellt.
        </p>
        <p>
        Hinweis: Die Daten werden nicht mehr geprüft.
        </p>
        <p>
        Satzaufbau für die Daten eines Schülers:
        <ol>
          <li>Länge (= Anzahl der Zeichen) des nachfolgenden 
              Namens (int-Zahl)</li>
          <li>Name (Zeichenkette mit variabler Länge)</li>
          <li>Notendurchschnitt (float-Zahl)</li>
          <li>Klasse (Zeichenkette mit fixer Länge)</li>
        </ol>
        </p>
        </summary>
        <remarks>
            <strong>
            Zahlen sind verkehrt gespeichert (Little-Endian)!
            </strong>
        </remarks>

        <returns>
          true, wenn das Lesen des Datensatzes erfolgreich war
        </returns>
        */
        public bool Lesen()
        {
            String  name;
            float   notenschnitt;
            String  klasse;

            byte[]  bytes;
            char[]  chars;
            int     anzahl;

            try
            {
                // Länge des 1. Datenfeldes
                anzahl = eingabesatz.ReadInt32();

                // 1. Datenfeld (Name - Zeichenkette mit variabler Länge)
                bytes = eingabesatz.ReadBytes(anzahl);
                chars = new char[bytes.Length];
                Array.Copy(bytes, chars, bytes.Length);
                name = new String(chars);

                // 2. Datenfeld (Notendurchschnitt - Zahl)
                notenschnitt = eingabesatz.ReadSingle();

                // 3. Datenfeld (Klassenbezeichnung - Zeichenkette fixer Länge)
                bytes = eingabesatz.ReadBytes(Schueler.KLASSELEER.Length);
                chars = new char[bytes.Length];
                Array.Copy(bytes, chars, bytes.Length);
                klasse = new String(chars);
            }
            catch (EndOfStreamException exc)
            {
                eof = true;
                Console.WriteLine("EOF");  // diese Ausgabe dient nur zum Testen
                return  true;     // EOF ist kein Fehler
            }
            catch (IOException fehler)
            {
                Console.WriteLine("Fehler beim Lesen des Datensatzes "
                                  + fehler.Message);           // nur zum Testen
                return  false;
            }

            // --------------------
            // Daten wurden korrekt gelesen -> Objekt für Datensatz instanzieren

            schueler = new Schueler();
            schueler.Name = name;
            schueler.Notenschnitt = notenschnitt;
            schueler.Klasse = klasse;

            return  true;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Liest den nächsten Datensatz aus der Datei (im C#-Format).<br />
        <p>
        Die Datenfelder werden (entsprechend dem Satzaufbau) der Reihe nach aus 
        der Datei gelesen.<br />
        Bei einem Fehler wird kein Datensatz erstellt.
        </p>
        <p>
        Hinweis: Die Daten werden nicht mehr geprüft.
        </p>
        <p>
        Zeichenketten sind entsprechend ihrer Länge in einem C#-spezifischen 
        Format gespeichert.
        </p>
        </summary>
        <remarks>
            <strong>
            Zahlen sind verkehrt gespeichert (Little-Endian)!
            </strong>
        </remarks>

        <returns>
          true, wenn das Lesen des Datensatzes erfolgreich war
        </returns>
        */
        public bool LesenCSproprietaer()
        {
            String  name;
            float   notenschnitt;
            String  klasse;

            try
            {
                // 1. Datenfeld (Name - Zeichenkette mit variabler Länge)
                name = eingabesatz.ReadString();

                // 2. Datenfeld (Notendurchschnitt - Zahl)
                notenschnitt = eingabesatz.ReadSingle();

                // 3. Datenfeld (Klassenbezeichnung - Zeichenkette fixer Länge)
                klasse = eingabesatz.ReadString();
            }
            catch (EndOfStreamException exc)
            {
                eof = true;
                Console.WriteLine("EOF");  // diese Ausgabe dient nur zum Testen
                return  true;     // EOF ist kein Fehler
            }
            catch (IOException fehler)
            {
                Console.WriteLine("Fehler beim Lesen des Datensatzes "
                                  + fehler.Message);           // nur zum Testen
                return  false;
            }

            // --------------------
            // Daten wurden korrekt gelesen -> Objekt für Datensatz instanzieren

            schueler = new Schueler();
            schueler.Name = name;
            schueler.Notenschnitt = notenschnitt;
            schueler.Klasse = klasse;

            return  true;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Schließt die Datei.
        </summary>
        <remarks>
        Close löst keine Ausnahmen aus.
        </remarks>
        */
        public void Schliessen()
        {
            eingabesatz.Close();
        }
    }
}
