/*
 * DAOschreibenSchueler.cs
 */
using System;
using System.IO;
using System.Text;

using Beringer.oopBuch.ModelKlassen;

namespace Beringer.oopBuch.DaoKlassen
{
    /**
    <summary>
    DAO-Klasse für das Schreiben von Schülerdaten in eine Datei 
    (Nichtstandarddatei).
    <p>
    Die Schülerdaten sind die Attribute eines 'Schueler'-Objekts.
    </p>

    <p>
    Seit: November 2004 <br />
    Änderung: Juni 2007 (Refaktorierung) <br />
    Änderung: Juli 2012 (FileStream-Objekt als lokale Variable) <br />
    Änderung: August 2014 (neue Namensbereiche 'ModelKlassen' und 'DaoKlassen')
    </p>

    <seealso cref="Schueler" />
    <seealso cref="DAOerfassenSchueler" />
    <seealso cref="DAOausgebenSchueler" />
    <seealso cref="Beringer.oopBuch.EinAusgabe.ErfassenSchreiben" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    <ul>
      <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
      <li>Die Konsolausgaben bei Fehlern dienen nur zur Kontrolle und sollten 
          durch Erzeugung eigener Fehlerobjekte ersetzt werden.</li>
    </ul>
    </remarks>
    */
    public class DAOschreibenSchueler
    {
        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        /// Datenobjekt für die zu schreibenden Daten
        private Schueler  schueler;

        /// Verbindungsobjekt zur Datei
        private BinaryWriter  ausgabesatz;

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Set-Property für die Daten des zu schreibenden Datensatzes.
        </summary>
        */
        public Schueler SchuelerDaten
        {
            set
            {
                schueler = value;
            }
        }

        // ---------------------------------------------------------------------
        // Instanzmethoden für Dateiverarbeitung
        // ---------------------------------------------------------------------

        /**
        <summary>
        Öffnet eine (Nichtstandard-)Datei zum Schreiben.
        </summary>

        <param name="dateiname">
          Name der zu öffnenden Datei (als Zeichenkette)
        </param>

        <returns>
          true, wenn das Öffnen der Datei erfolgreich war
        </returns>
        */
        public bool Oeffnen(String dateiname)
        {
            FileStream  ausgabedatei;               // entfällt bei Schachtelung

            try
            {
                ausgabedatei = new FileStream(dateiname, FileMode.Create);
                ausgabesatz  = new BinaryWriter(ausgabedatei);

// Schachtelung ist möglich
//              ausgabesatz = new BinaryWriter(
//                                  new FileStream(dateiname, FileMode.Create));
            }
            catch (IOException fehler)
            {
                Console.WriteLine("Fehler beim Öffnen der Datei: "
                                  + fehler.Message);           // nur zum Testen
                return  false;
            }

            return  true;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Schreibt einen neuen Datensatz in die Datei.
        <p>
        Die Datenfelder werden (entsprechend dem Satzaufbau) der Reihe nach in 
        die Datei geschrieben.
        </p>
        <p>
        Satzaufbau für die Daten eines Schülers:
        <ol>
          <li>Länge (= Anzahl der Zeichen) des nachfolgenden 
              Namens (int-Zahl)</li>
          <li>Name (Zeichenkette mit variabler Länge)</li>
          <li>Notendurchschnitt (float-Zahl)</li>
          <li>Klasse (Zeichenkette mit fixer Länge)</li>
        </ol>
        </p>
        </summary>
        <remarks>
            <strong>
            Die Speicherung von Zahlen erfolgt verkehrt (Little-Endian)!
            </strong>
        </remarks>

        <returns>
           true, wenn das Schreiben des Datensatzes erfolgreich war
        </returns>
        */
        public bool Schreiben()
        {
            byte []   bytes;
            Encoding  enc = Encoding.Default;

            try
            {
                // Speichern der Länge des 1. Datenfeldes
                ausgabesatz.Write( (int) schueler.Name.Length );

                // 1. Datenfeld (Name - Zeichenkette mit variabler Länge)
                bytes = enc.GetBytes(schueler.Name);
                ausgabesatz.Write( bytes );

                // 2. Datenfeld (Notendurchschnitt - Zahl)
                ausgabesatz.Write( (float) schueler.Notenschnitt );

                // 3. Datenfeld (Klassenbezeichnung - Zeichenkette fixer Länge)
                bytes = enc.GetBytes(schueler.Klasse);
                ausgabesatz.Write( bytes );
            }
            catch (IOException fehler)
            {
                Console.WriteLine("Fehler beim Schreiben des Datensatzes: "
                                  + fehler.Message);           // nur zum Testen
                return  false;
            }
            return  true;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Schreibt einen neuen Datensatz in die Datei (im C#-Format).
        <p>
        Zeichenketten werden automatisch mit ihrer Länge in einem 
        C#-spezifischen Format geschrieben.
        </p>
        </summary>
        <remarks>
            <strong>
            Die Speicherung von Zahlen erfolgt verkehrt (Little-Endian)!
            </strong>
        </remarks>

        <returns>
          true, wenn das Schreiben des Datensatzes erfolgreich war
        </returns>
        */
        public bool SchreibenCSproprietaer()
        {
            try
            {
                // 1. Datenfeld (Name - Zeichenkette mit variabler Länge)
                ausgabesatz.Write( schueler.Name );
                           /*
                             schreibt zuerst die Anzahl der Zeichen 
                                                      (pro 127 Zeichen 1 Byte!),
                             schreibt danach UTF-8-Zeichen (8 Bit)
                           */

                // 2. Datenfeld (Notendurchschnitt - Zahl)
                ausgabesatz.Write( (float) schueler.Notenschnitt );

                // 3. Datenfeld (Klassenbezeichnung - Zeichenkette fixer Länge)
                ausgabesatz.Write( schueler.Klasse );
                           /*
                             schreibt zuerst die Anzahl der Zeichen 
                                                      (pro 127 Zeichen 1 Byte!),
                             schreibt danach UTF-8-Zeichen (8 Bit)
                           */
            }
            catch (IOException fehler)
            {
                Console.WriteLine("Fehler beim Schreiben des Datensatzes: "
                                  + fehler.Message);           // nur zum Testen
                return  false;
            }
            return  true;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Schließt die Datei.
        </summary>
        <remarks>
        Close löst keine Ausnahmen aus.
        </remarks>
        */
        public void Schliessen()
        {
            ausgabesatz.Close();
        }
    }
}
