/*
 * Auto.cs
 */
using System;

namespace Beringer.oopBuch.EinfacheBsp
{
    /**
    <summary>
    Eine sehr einfache Auto-Klasse, jede Instanz dieser Klasse repräsentiert 
    ein Auto.
    <ul>
      <li>Jede Instanz wird mit einer Anfangsgeschwindigkeit von 10 km/h 
          initialisiert.</li>
      <li>Rauf- und runterschalten bewirkt keine Geschwindigkeitsänderung.</li>
      <li>Bremsen und beschleunigen bewirkt immer eine Geschwindigkeitsänderung 
          von 10%, jedoch keinen Gangwechsel.</li>
    </ul>

    <p>
    Seit: November 2004 <br />
    Version: Juli 2012 (PKW-Klasse umbenennen auf 'Auto'.
                        Keine Initialisierung des Besitzers im Konstruktor.)
    </p>

    <seealso cref="Person" />
    <seealso cref="AutoTesten" />
    <seealso cref="Beringer.oopBuch.EinfacheBspProgramme.ProgrammAutoTesten"/>
    <seealso cref="AutoM" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    Es wurde zusätzlich noch (neben der Methode 'ZeigeInhalt') 
    die Methode 'ToString' überschrieben.
    <p>
    Diese Klasse dient auch als Fachobjekt ("model") für die GUI-Beispiele.
    </p>
    </remarks>
    */
    public class Auto
    {
        // ---------------------------------------------------------------------
        // Klassenattribute
        // ---------------------------------------------------------------------

        /// <summary>Mehrwertsteuersatz</summary>
        private static int  mwstSatz;

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private String  farbe;
        private int     anzahlGaenge;
        private float   geschwindigkeit;
        private int     gang;
        private Person  besitzer;

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Standardkonstruktor.
        </summary>
        <remarks>
        Alle Attribute (ausgenommen Besitzer) werden mit fixen Werten 
        initialisiert.
        Es wird eine Anfangsgeschwindigkeit von 10 km/h angenommen.
        </remarks>
        */
        public Auto()
        {
            Farbe = "rot";
            AnzahlGaenge = 5;
            Geschwindigkeit = 10.0f;
            Gang = 1;
        }

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Property für die Farbe des Autos.
        </summary>
        */
        public String Farbe
        {
            get
            {
                return farbe;
            }
            set
            {
                farbe = value;
            }
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Property für die Maximalanzahl der Gänge des Autos.
        </summary>
        <remarks>
        Die Änderung der Maximalanzahl der Gänge des Autos entspricht einem 
        Umbau des Autos.
        <p>
        Set-Property:
        <ul>
          <li>Die Maximalanzahl der Gänge muss zumindest 1 sein.</li>
          <li>Der eingelegte Gang wird auf 1 zurückgesetzt.</li>
        </ul>
        </p>
        </remarks>
        */
        public int AnzahlGaenge
        {
            get
            {
                return anzahlGaenge;
            }
            set
            {
                if (value >= 1)
                {
                    anzahlGaenge = value;
                }
                Gang = 1;
            }
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Property für die aktuelle Geschwindigkeit.
        </summary>
        <remarks>
        Set-Methode: Es erfolgt aber keine Prüfung auf Plausibilität!
        </remarks>
        */
        public float Geschwindigkeit
        {
            get
            {
                return geschwindigkeit;
            }
            set
            {
                geschwindigkeit = value;
            }
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Property für den gerade eingelegten Gang.
        </summary>
        <remarks>
        Set-Methode: Es erfolgt eine Prüfung auf Gültigkeit!
        <p>
        Der eingelegte Gang kann nicht kleiner sein als 1 und nicht höher als 
        die Anzahl der Gänge. <br />
        Bei einem Fehler wird der Gang nicht geändert.
        </p>
        </remarks>
        */
        public int Gang
        {
            get
            {
                return gang;
            }
            set
            {
                if ((value >= 1) && (value <= anzahlGaenge))
                {
                    gang = value;
                }
            }
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Property für den Besitzer des Autos.
        </summary>
        */
        public Person Besitzer
        {
            get
            {
                return besitzer;
            }
            set
            {
                besitzer = value;
            }
        }

        // ---------------------------------------------------------------------
        // Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Gang um 1 erhöhen.
        </summary>
        */
        public void Raufschalten()
        {
            if (gang < anzahlGaenge)
            {
                gang = gang + 1;
            }
        }

        // ----------

        /**
        <summary>
        Gang um 1 vermindern.
        </summary>
        */
        public void Runterschalten()
        {
            if (gang > 1)
            {
                gang = gang - 1;
            }
        }

        // ----------

        /**
        <summary>
        Geschwindigkeit um 10% erhöhen.
        </summary>
        */
        public void Beschleunigen()
        { 
            geschwindigkeit = geschwindigkeit * 1.1f;
        }

        // ----------

        /**
        <summary>
        Geschwindigkeit um 10% vermindern.
        </summary>
        */
        public void Bremsen()
        {
            geschwindigkeit = geschwindigkeit * 0.9f;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Schreibt alle Attribute in die Standardausgabe. <br />
        Achtung: Besitzer sollte bereits initialisiert sein!
        </summary>
        <remarks>
        Anstelle dieser Methode sollte besser die Methode 'Object.ToString()' 
        überschrieben werden.
        </remarks>
        */
        public void ZeigeInhalt()
        {
            Console.WriteLine(GetType().FullName 
                         + "-Daten: [ Mehrwertsteuersatz = " + mwstSatz + "%" );
            Console.WriteLine("     Besitzer = " + besitzer.Heisst()
                                  + ", Farbe = " + farbe
                          + ", Anzahl Gaenge = " + anzahlGaenge );
            Console.WriteLine("     eingelegter Gang = " + gang
                  + " / aktuelle Geschwindigkeit = " + geschwindigkeit + " ]" );
        }

        // ---------------------------------------------------------------------
        // Methode 'ToString'
        // ---------------------------------------------------------------------

        /**
        <summary>
        Überschreiben der Methode 'String Object.ToString()'.
        <p>
        Achtung: Besitzer sollte bereits initialisiert sein!
        </p>
        </summary>
        <remarks>
        Liefert eine beschreibende Text-Darstellung einer Instanz der Klasse:
        <pre>
          Auto-Daten: [ Mehrwertsteuersatz = ...%
               Besitzer = ..., Farbe = ..., Anzahl Gaenge = ...
               eingelegter Gang = ... / aktuelle Geschwindigkeit = ... ]
        </pre>
        <p>
        Hinweis: Statt der Klasse 'String' sollte die Klasse 'StringBuilder' 
                 verwendet werden!
        </p>

        <seealso cref="System.Object#toString()" />

        </remarks>

        <returns>
          Zeichenkette, die eine Instanz beschreibt
        </returns>
        */
        override public String ToString()
        {
            String  neueZeile = Environment.NewLine;

            // Zugriff auf Attribute nur über Zugriffsmethoden und Properties 
            // (Datenkapselung):
            return  GetType().FullName 
             + "-Daten: [ Mehrwertsteuersatz = " + MwstSatz() + "%"  + neueZeile
             + "     Besitzer = "                + Besitzer.Heisst()
             + ", Farbe = "                      + Farbe
             + ", Anzahl Gaenge = "              + AnzahlGaenge      + neueZeile
             + "     eingelegter Gang = "        + Gang
             + " / aktuelle Geschwindigkeit = "  + Geschwindigkeit + " ]" ;
        }

        // ---------------------------------------------------------------------
        // Klassenmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert den Mehrwertsteuersatz.
        </summary>

        <returns>
          Mehrwertsteuersatz
        </returns>
        */
        public static int MwstSatz()
        {
            return  mwstSatz;
        }

        // ----------

        /**
        <summary>
        Ändert den Mehrwertsteuersatz.
        </summary>

        <param name="value">
          neuer Mehrwertsteuersatz
        </param>
        */
        public static void MwstSatz(int value)
        {
            mwstSatz = value;
        }
    }
}
