/*
 * Datum.cs
 */
using System;

namespace Beringer.oopBuch.EinfacheBsp
{
    /**
    <summary>
    Eine einfache Datumsklasse.

    <p>
    Seit: November 2004
    </p>

    <seealso cref="DatumTesten" />
    <seealso cref="Beringer.oopBuch.EinfacheBspProgramme.ProgrammDatumTesten"/>
    <seealso cref="System.DateTime" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    Instanzen dieser Klasse können kopiert werden.
    </remarks>
    */
    public class Datum : ICloneable
    {
        // ---------------------------------------------------------------------
        // Klassenkonstante
        // ---------------------------------------------------------------------

        private static readonly String[]  Monatsnamen  = { "Jänner   ",
                                                           "Februar  ",
                                                           "März     ",
                                                           "April    ",
                                                           "Mai      ",
                                                           "Juni     ",
                                                           "Juli     ",
                                                           "August   ",
                                                           "September",
                                                           "Oktober  ",
                                                           "November ",
                                                           "Dezember "  };

        private static readonly int[]     TageProMonat
                = { 31,  28,  31,  30,  31,  30,  31,  31,  30,  31,  30,  31 };

        private static readonly int[]     LfdTagMonatsanfang
                = {  1,  32,  60,  91, 121, 152, 182, 213, 244, 274, 305, 335 };

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private int  jj;
        private int  mm;
        private int  tt;

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Standardkonstruktor.
        </summary>
        <remarks>
        Initialisierung mit aktuellem Tagesdatum
        </remarks>
        */
        public Datum()
        {
            DateTime  heute = DateTime.Today;    // liefert aktuelles Tagesdatum

            jj = heute.Year;
            mm = heute.Month;
            tt = heute.Day;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Initialisierung mit einem beliebigen Tagesdatum.
        </summary>
        <remarks>
        Dieses Datum wird auf Gültigkeit geprüft, bei einem Fehler wird das 
        aktuelle Tagesdatum zur Initialisierung verwendet.
        </remarks>

        <param name="jahr">
          Jahr
        </param>
        <param name="monat">
          Monatsnummer (bei 1 (= Jänner) beginnend)
        </param>
        <param name="tag">
          Tag
        </param>
        */
        public Datum(int jahr, int monat, int tag)
               : this()               // initialisieren mit aktuellem Tagesdatum
        {
            setDatum(jahr, monat, tag);
        }

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Get-Property für das Jahr.
        </summary>
        */
        public int JJ
        {
            get
            {
                return jj;
            }
        }

        // ----------

        /**
        <summary>
        Get-Property für die Monatsnummer.
        </summary>
        */
        public int MM
        {
            get
            {
                return mm;
            }
        }

        // ----------

        /**
        <summary>
        Get-Property für den Tag.
        </summary>
        */
        public int TT
        {
            get
            {
                return tt;
            }
        }

        // ----------

        /**
        <summary>
        Get-Property für den Monatsnamen entsprechend der Monatsnummer.
        </summary>
        */
        public String Monatsname
        {
            get
            {
                return Monatsnamen[mm-1];
            }
        }

        // ---------------------------------------------------------------------
        // Private Set-Methoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Prüft und setzt ein korrektes Datum.
        </summary>
        <remarks>
        Der Einfachheit halber werden Fehlermeldungen auf die Konsole 
        geschrieben.
        </remarks>

        <param name="jahr">
          Jahr
        </param>
        <param name="monat">
          Monatsnummer (bei 1 (= Jänner) beginnend)
        </param>
        <param name="tag">
          Tag
        </param>

        <returns>
          true, wenn das Datum korrekt ist
        </returns>
        */
        private bool setDatum(int jahr, int monat, int tag)
        {
            int tageProMonat;

            if (jahr == 0)
            {
                Console.WriteLine("Fehler: " + jahr + " ist ungültig");
                return false;
            }

            if ((monat < 1) || (monat > 12))
            {
                Console.WriteLine("Fehler: " + monat + " ist ungültig");
                return false;
            }

            tageProMonat = TageProMonat[monat - 1];
            if ((monat == 2) && (Datum.IstSchaltjahr(jahr)))
            {
                tageProMonat++;
            }
            if ((tag < 1) || (tag > tageProMonat))
            {
                Console.WriteLine("Fehler: " + tag + " ist ungültig");
                return false;
            }

            jj = jahr;
            mm = monat;
            tt = tag;

            return true;
        }

        // ---------------------------------------------------------------------
        // Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert den laufenden Tag des Jahres.
        </summary>

        <returns>
          laufender Tag des Jahres
        </returns>
        */
        public int LfdTagDesJahres()
        {
            int  lfd;

            lfd = LfdTagMonatsanfang[mm-1] + tt - 1;
            if ((mm > 2) && (this.IstSchaltjahr()))
            {
                lfd++;
            }
            return  lfd;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Prüft das eigene Jahr auf Schaltjahr.
        </summary>

        <returns>
          true, wenn das eigene Jahr ein Schaltjahr ist
        </returns>
        */
        public bool IstSchaltjahr()
        {
            return  Datum.IstSchaltjahr(jj);
        }

        // ---------------------------------------------------------------------
        // Methoden 'ToString' und 'Clone'
        // ---------------------------------------------------------------------

        /**
        <summary>
        Überschreiben der Methode 'String Object.toString()'.
        </summary>
        <remarks>
        Liefert eine beschreibende Text-Darstellung einer Instanz der Klasse:
        <pre>
          TT. MM. JJJJ
        </pre>

        <seealso cref="System.Object#ToString()" />

        </remarks>

        <returns>
          Datum in der Form "TT. MM. JJJJ"
        </returns>
        */
        override public String ToString()
        {
            return  TT + ". " + MM + ". " + JJ;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Abstrakte Methode 'Clone()' von 'ICloneable' definieren.
        <p>
        Kopieren einer Datum-Instanz (tiefe Kopie):
        <ol>
          <li>Sichtbarkeit auf 'public' anheben ('ICloneable.Clone()' ist 
              'protected'!).</li>
          <li>Vererbte Methode 'Object.MemberwiseClone()' aufrufen 
              ('base.MemberwiseClone()').</li>
          <li>Veränderliche Attribute kopieren: <br />
              Da diese Klasse keine veränderlichen Attribute hat, ist für eine
              tiefe Kopie nichts weiter zu tun. <br />
              (Wenn bereits in get-/set-Properties nur Kopien von 
               veränderlichen Attributen verwendet werden, könnten diese 
               Properties für das Kopieren dieser Attribute verwendet 
               werden.</li>
        </ol>

        <seealso cref="System.Object#MemberwiseClone()" />

        </summary>

        <returns>
          (tiefe) Kopie eines Datums
        </returns>
        */
        public Object Clone()
        {
            Datum  datumKopie;

            datumKopie = (Datum) base.MemberwiseClone();

            /* hier folgt das Kopieren der veränderlichen Attribute (für eine 
               tiefe Kopie) z.B.: kopie.Attribut = Attribut;
            */

            return  datumKopie;
        }

        // ---------------------------------------------------------------------
        // Klassenmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert das aktuelle Tagesdatum.
        </summary>

        <returns>
          Instanz des aktuellen Tagesdatums
        </returns>
        */
        public static Datum Heute()
        {
            return  new Datum();
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Prüft ein vorgegebenes Jahr auf Schaltjahr.
        </summary>

        <param name="jahr">
          zu prüfende Jahreszahl
        </param>

        <returns>
          true, wenn 'jahr' ein Schaltjahr ist
        </returns>
        */
        public static bool IstSchaltjahr(int jahr)
        {
            return 
                 ((jahr%4 != 0) || ((jahr%100 == 0) && (jahr%400 != 0))) ? false
                                                                         : true;
        }
    }
}
