/*
 * Person.cs
 */
using System;

namespace Beringer.oopBuch.EinfacheBsp
{
    /**
    <summary>
    Eine einfache Personen-Klasse.
    <p>
    Jede Instanz dieser Klasse repräsentiert eine natürliche Person.
    </p>
    <ul>
      <li>Klassenkonstante für gültige Geschlechtskennzeichen.</li>
      <li>Jede Person hat immer ein Geburtsjahr und ein Geschlecht.</li>
      <li>Geburtsjahr und Geschlecht können nicht geändert werden.</li>
      <li>Geburtsjahr und Geschlecht werden geprüft: <br />
          Geburtsjahr darf nicht größer sein als das laufende Jahr,
          das Geschlecht darf nur 'M' oder 'W' sein.</li>
    </ul>

    <p>
    Seit: November 2004 <br />
    Änderung: Juni 2006 (Geburtsjahr)
    </p>

    <seealso cref="Beringer.oopBuch.Organisation.Person" />
    <seealso cref="Auto" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    Die Standardzugriffsmethoden wurden NICHT als Properties definiert.<br />
    Die Namen dieser Standardzugriffsmethoden entsprechen auch NICHT den 
    üblichen UML-Konventionen ('getAttribut' bzw. 'setAttribut'). 
    </remarks>
    */
    public class Person
    {
        // ---------------------------------------------------------------------
        // Klassenkonstante
        // ---------------------------------------------------------------------

        /**
        <summary>
        Kennzeichen für männliches Geschlecht.
        </summary>
        <remarks>
        Der Wert könnte von externen Datenquellen geholt werden, wird jedoch 
        der Einfachheit halber fix vorgegeben.
        </remarks>
        */
        public static readonly char  MAENNLICH = 'M';

        /**
        <summary>
        Kennzeichen für weibliches Geschlecht.
        </summary>
        <remarks>
        Der Wert könnte von externen Datenquellen geholt werden, wird jedoch 
        der Einfachheit halber fix vorgegeben.
        </remarks>
        */
        public static readonly char  WEIBLICH  = 'W';

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private String  vorname;
        private int     geburtsjahr;
        private char    geschlecht;

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Jede Person muss ein Geburtsjahr und ein Geschlecht haben.
        </summary>
        <remarks>
        Standardkonstruktor ist nicht erlaubt.
        </remarks>

        <param name="geburtsjahr">
          Geburtsjahr
        </param>
        <param name="geschlecht">
          Geschlecht
        </param>
        */
        public Person(int geburtsjahr, char geschlecht)
        {
            aendernGeburtsjahr(geburtsjahr);
            aendernGeschlecht(geschlecht);
        }

        // ---------------------------------------------------------------------
        // (Öffentliche) Zugriffsmethoden (Get- und Set-Methoden)
        // In C# sollten dies Properties sein!
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert den Vornamen der Person.
        </summary>

        <returns>
          Vorname
        </returns>
        */
        public String Heisst()
        {
            return  vorname;
        }

        // ----------

        /**
        <summary>
        Ändert den Vornamen der Person.
        </summary>

        <param name="neuerVorname">
          neuer Vorname
        </param>
        */
        public void AendernVorname(String neuerVorname)
        {
            vorname = neuerVorname;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert das Geburtsjahr der Person.
        </summary>

        <returns>
          Geburtsjahr
        </returns>
        */
        public int Geboren()
        {
            return  geburtsjahr;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert das Geschlecht der Person.
        </summary>

        <returns>
          Geschlecht
        </returns>
        */
        public char HatGeschlecht()
        {
            return  geschlecht;
        }

        // ---------------------------------------------------------------------
        // Sonstige Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert die Anrede für die Person (abhängig vom Geschlecht).
        </summary>
        <remarks>
        Die Anrede ist fix codiert und nicht variabel!
        </remarks>

        <returns>
          Anrede entsprechend dem Geschlecht
        </returns>
        */
        public String Anrede()
        {
            if (geschlecht == MAENNLICH)
            {
                return  "Sehr geehrter Herr";
            }

            return  "Sehr geehrte Frau";

            /* 
              kürzer mit Bedingungsoperator:

            return  (geschlecht == MAENNLICH) ? "Sehr geehrter Herr"
                                              : "Sehr geehrte Frau";
            */
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Berechnet das Alter der Person.
        </summary>

        <returns>
          Alter in Jahren
        </returns>
        */
        public int AlterBerechnen()
        {
            return  DateTime.Today.Year - geburtsjahr;
        }

        // ---------------------------------------------------------------------
        // Nichtöffentliche Zugriffsmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Setzt das Geburtsjahr der Person.
        </summary>
        <remarks>
        Das Geburtsjahr wird auf Gültigkeit geprüft:
            (Geburtsjahr <= laufendes Jahr). <br />
        Bei einem Fehler wird das Geburtsjahr nicht gesetzt!
        </remarks>

        <param name="einJahr">
          neues Geburtsjahr
        </param>

        <returns>
          true, wenn 'einJahr' ein gültiges Geburtsjahr ist
        </returns>
        */
        private bool aendernGeburtsjahr(int einJahr)
        {
            int lfdJahr = DateTime.Today.Year;

            if (einJahr <= lfdJahr)
            {
                geburtsjahr = einJahr;
                return  true;
            }

            // ungültiges Geburtsjahr
            return  false;
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Setzt das Geschlecht der Person.
        </summary>
        <remarks>
        Das Geschlecht wird auf Gültigkeit geprüft. <br />
        Bei einem Fehler wird das Geschlecht nicht gesetzt!
        </remarks>

        <param name="kennzeichen">
          Kennzeichen für Geschlecht (männlich oder weiblich)
        </param>

        <returns>
          true, wenn 'kennzeichen' ein gültiges Geschlechtskennzeichen ist
        </returns>
        */
        private bool aendernGeschlecht(char kennzeichen)
        {
            if ( (kennzeichen == MAENNLICH) || (kennzeichen == WEIBLICH) )
            {
                geschlecht = kennzeichen;
                return  true;
            }

            // ungültiges Kennzeichen für Geschlecht
            return  false;
        }
    }
}
