/*
 * DialogModalElternfensterV.cs
 */
using System;
using System.Windows.Forms;
using System.Drawing;

using Beringer.oopBuch.EinfacheBsp;

namespace Beringer.oopBuch.GUI
{
    /**
    <summary>
    Hauptfenster für einen modalen Dialog. <br />
    Schaltfläche öffnet Dialogfenster, 
    Textbereich speichert Dialogeingabe.
    <p>
    Model-Klasse: 'AutoM'
                  (dient als Fachobjekt zum Testen dieser Fensterklasse)
    </p>

    <p>
    Seit: August 2005
    </p>

    <seealso cref="DialogModalKindfensterV" />
    <seealso cref="DialogModalProgramm" />
    <seealso cref="Beringer.EinfacheBsp.AutoM" />

    Autor: Beringer Alfred
    </summary>
    <remarks>
    Man beachte, dass für die Internationalisierung dieser Klasse für alle 
    Texte bzw. für alle Beschriftungen bereits Klassenkonstante definiert 
    wurden, dass aber diese der Einfachheit halber im Klassenkonstruktor fix 
    vorgegeben sind. <br />
    Texte für die Schaltflächen im Dialogfenster werden nicht benötigt.
    </remarks>
    */
    public class DialogModalElternfensterV : Form
    {
        // ---------------------------------------------------------------------
        // Klassenkonstante
        // ---------------------------------------------------------------------

        private static readonly String  NeueZeile = Environment.NewLine;

        /// Text für Titelzeile
        private static readonly String  FENSTERTITEL;

        /// Beschriftung zu Schaltfläche1 'Dialog öffnen'
        private static readonly String  LABEL1TEXT;

        /// Text für Schaltfläche1 'Dialog öffnen'
        private static readonly String  BUTTON;

        /// Beschriftung zu Textbereich
        private static readonly String  LABEL2TEXT;

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private Label    label1 = new Label();
        private Button   button = new Button();
        private Label    label2 = new Label();
        private TextBox  text   = new TextBox();

        private DialogModalKindfensterV  dlg;

        /// Fachobjekt (zum Testen)
        private AutoM  auto = new AutoM();

        // ---------------------------------------------------------------------
        // Klassenkonstruktor
        // ---------------------------------------------------------------------

        /**
        <summary>
        Klassenkonstruktor. <br />
        Initialisieren der Klassenkonstanten. <br />
        Die Anfangswerte für die Klassenkonstanten werden im Allg. von 
        externen Datenquellen übernommen (z.B. von einer Ressource-Datei).
        </summary>
        <remarks>
        Der Einfachheit halber werden die Anfangswerte fix vorgegeben.
        </remarks>
        */
        static DialogModalElternfensterV()
        {
            FENSTERTITEL = "Dialog modal - Elternfenster";
            LABEL1TEXT   = "Öffnet Dialogfenster: ";
            BUTTON       = "Dialog";
            LABEL2TEXT   = "Eingabe im Dialogfenster war: ";
        }

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Standardkonstruktor.
        </summary>
        */
        public DialogModalElternfensterV()
        {
            this.Text = FENSTERTITEL;
            this.Size = new Size(440, 240);

            this.CenterToScreen();
            this.AcceptButton = button;        // Standardschaltfläche

            // GUI-Komponenten mit Texten initialisieren

            label1.Text = LABEL1TEXT;
            button.Text = BUTTON;
            label2.Text = LABEL2TEXT;

            // Schaltfläche

            button.AutoSize = true;
            button.Anchor = AnchorStyles.None | AnchorStyles.Left;
            button.Margin = new Padding(0, 20, 0, 20);

            // Beschriftungen (Labels) rechtsbündig platzieren

            label1.AutoSize = true;
            label1.Anchor = AnchorStyles.None | AnchorStyles.Right;

            label2.AutoSize = true;
            label2.Anchor = AnchorStyles.Top | AnchorStyles.Right;

            // mehrzeiliges Textfeld

            text.ReadOnly = true;
            text.Multiline = true;
            text.WordWrap = false;
            text.ScrollBars = ScrollBars.Both;
            text.Size = new Size(200, 80);

            // Tabellen-Layout für Fensterinhalt definieren (2 Spalten)

            TableLayoutPanel pmitte = new TableLayoutPanel();
            pmitte.AutoSize = true;
            pmitte.ColumnCount = 2;
            pmitte.Dock = DockStyle.Top;
            pmitte.Padding = new Padding(20);

            // GUI-Komponenten der Windows-Tabellenform hinzufügen
            // (von links nach rechts (zeilenweise - je 2 Spalten)

            pmitte.Controls.Add(label1);
            pmitte.Controls.Add(button);
            pmitte.Controls.Add(label2);
            pmitte.Controls.Add(text);

            // GUI-Komponenten der Windows-Form hinzufügen

            this.Controls.Add(pmitte);

            // Ereignisse für GUI-Komponenten definieren

            button.Click += buttonevent;

            initDialogfenster();
        }

        // ---------------------------------------------------------------------
        // private Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Dialogfenster erzeugen - eigene Referenz mitgeben, falls Daten vom 
                                 Elternfenster benötigt werden
        </summary>
        <remarks>
        Fenster als Dialog darstellen, d.h. 
        Fenstergröße kann nicht verändert werden, nur Schließen möglich
        </remarks>
        */
        private void initDialogfenster()
        {
            dlg = new DialogModalKindfensterV(this);
            dlg.FormBorderStyle = FormBorderStyle.Fixed3D;
            dlg.MinimizeBox = false;
            dlg.MaximizeBox = false;
        }

        // *********************************************************************
        // Ereignisbehandlungsmethoden
        // *********************************************************************

        /**
        <summary>
        Delegate-Methode für Ereignis.
        </summary>

        <param name="sender">
          Ereignisquelle
        </param>
        <param name="e">
          Ereignisobjekt (enthält Informationen über das Ereignis)
        </param>
        */
        private void buttonevent(Object sender, EventArgs e)
        {
            
            dlg.ShowDialog();                          // Dialogfenster anzeigen
                            // Bem.: Methode 'Show()' öffnet nichtmodalen Dialog

            /*
             * Applikation stoppt hier (Dialog ist modal!)
             */

            if (dlg.DialogResult == DialogResult.OK)
            {
                auto.Farbe = dlg.Rueckgabe;
                auto.ZeigeInhalt();                              // nur zum Testen

                text.AppendText(dlg.Rueckgabe);
                text.AppendText(NeueZeile);
            }
            this.BringToFront();
                                // Fenster bleibt manchmal im Hintergrund hängen
        }
    }
}
