/*
 * Schueler.cs
 */
using System;
using System.Text;

namespace Beringer.oopBuch.ModelKlassen
{
    /**
    <summary>
    Datenklasse ('Model'-Klasse) für eine Datenstruktur mit 3 Datenfeldern.
    <p>
    Diese Klasse dient als 'Model'-Klasse für die Demo-Programme zur 
    Verarbeitung von Dateien und Datenbanken.
    Objekte dieser Klasse (genauer: deren Attribute) werden als Datensätze in 
    einer Datei bzw. als Zeilen in einer Datenbank-Tabelle gespeichert.
    </p>
    <p>
    Jede Instanz dieser Klasse repräsentiert einen Schüler mit
    <ul>
       <li>Name</li>
       <li>Notendurchschnitt</li>
       <li>Klassenbezeichnung</li>
    </ul>
    </p>
    <p>
    Das 1. Datenfeld (Name) beinhaltet eine Zeichenkette, die beliebig lang sein
    kann (d.h. die Zeichenkette hat eine variable Länge). <br />
    Das 2. Datenfeld (Notendurchschnitt) beinhaltet einen float-Wert. <br />
    Das 3. Datenfeld (Klassenbezeichnung) beinhaltet eine Zeichenkette mit einer
    fixen Anzahl von Zeichen - in diesem Fall die Klassenbezeichnung mit immer 
    genau 7 Zeichen (d.h. die Zeichenkette hat eine fixe Länge).
    </p>
    <p>
    Der Einfachheit halber ist die 'toString'-Methode nicht internationalisiert.
    </p>

    <p>
    Seit: November 2004 <br />
    Änderung: Juni 2006 (Methode 'ToString' überschreiben) <br />
    Änderung: August 2014 (neue Namensbereiche 'ModelKlassen' und 'DaoKlassen')
    </p>

    <seealso cref="Beringer.oopBuch.DaoKlassen.DAOerfassenSchueler" />
    <seealso cref="Beringer.oopBuch.DaoKlassen.DAOschreibenSchueler" />
    <seealso cref="Beringer.oopBuch.DaoKlassen.DAOlesenSchueler" />
    <seealso cref="Beringer.oopBuch.DaoKlassen.DAOausgebenSchueler" />

    Autor: Beringer Alfred
    </summary>
    */
    public class Schueler
    {
        // ---------------------------------------------------------------------
        // Klassenkonstante
        // ---------------------------------------------------------------------

        /**
        <summary>
        Zeichenkette mit einer fixen Anzahl von Leerzeichen. Sie dient zum 
        Auffüllen einer zu kurzen Zeichenkette mit Leerzeichen.
        </summary>
        <remarks>
        Die Initialisierung erfolgt im Klassenkonstruktor.
        </remarks>
        */
        public static readonly String  KLASSELEER;

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private String  name;                // Zeichenkette mit variabler Länge
        private float   notenschnitt;
        private String  klasse;                  // Zeichenkette mit fixer Länge

        // ---------------------------------------------------------------------
        // Klassenkonstruktor
        // ---------------------------------------------------------------------

        /**
        <summary>
        Klassenkonstruktor (Klassen-Initialisierungsblock).
        </summary>
        <remarks>
        Die Werte für nicht initialisierte Klassenkonstante könnten von externen
        Datenquellen geholt werden (z.B. von einer Datei, einer Datenbank oder 
        von einer lokalisierten Resource-Datei).
		<p>
        Der Einfachheit halber werden die Werte fix vorgegeben - in diesem Fall 
        eine Zeichenkette mit genau 7 Leerzeichen.
		</p>
        </remarks>
        */
        static Schueler()
        {
            KLASSELEER = "       ";      // Zeichenkette mit genau 7 Leerzeichen
        }

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Property für das erste Datenfeld (Name) - eine Zeichenkette mit 
        variabler Länge.
        </summary>
        */
        public String Name
        {
            get
            {
                return  name;
            }
            set
            {
                name = value;
            }
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Property für das zweite Datenfeld (Notendurchschnitt) - ein float-Feld.
        </summary>
        */
        public float Notenschnitt
        {
            get
            {
                return  notenschnitt;
            }
            set
            {
                notenschnitt = value;
            }
        }

        // ---------------------------------------------------------------------

        /**
        <summary>
        Property für das dritte Datenfeld (Klassenbezeichnung) - eine 
        Zeichenkette fixer Länge.
        </summary>
        <remarks>
        Die Set-Methode kontrolliert auch gleichzeitig die Länge dieses 
        Feldes. <br />
        Eine zu kurze Zeichenkette wird mit Leerzeichen aufgefüllt, eine zu 
        lange wird einfach abgeschnitten..
        </remarks>
        */
        public String Klasse
        {
            get
            {
                return  klasse;
            }
            set
            {
                // mit Leerzeichen erweitern:
                klasse = value + KLASSELEER;
                // kürzen auf fixe Länge:
                klasse = klasse.Substring(0, KLASSELEER.Length);
            }
        }

        // ---------------------------------------------------------------------
        // Methode 'ToString'
        // ---------------------------------------------------------------------

        /**
        <summary>
        Überschreiben der Methode 'String Object.ToString()'.
        </summary>
        <remarks>
        Liefert eine beschreibende Text-Darstellung einer Instanz der Klasse:
        <pre>
          Schueler[ Name = ... (Länge = ...),
                    Notendurchschnitt = ...,
                    Klassenbezeichnung = *...* ]
        </pre>
        <p>
        Die Texte sind der Einfachheit halber Literale.
        </p>
        </remarks>

        <seealso cref="System.Object#toString() />

        <returns>
          Zeichenkette, die eine Instanz beschreibt
        </returns>
        */
        override public String ToString()
        {
            StringBuilder  text = new StringBuilder(this.GetType().Name);

            text.Append(
                 "[ Name = "               ).Append(name        ).Append(
                 " (Länge = "              ).Append(name.Length ).Append(
                "), Notendurchschnitt = "  ).Append(notenschnitt).Append(
                 ", Klassenbezeichnung = *").Append(klasse      ).Append("* ]");

            return  text.ToString();
        }
    }
}
