/*
 * Manager.cs
 */
using System;

namespace Beringer.oopBuch.Organisation
{
    /**
    <summary>
    Instanzen dieser Klasse repräsentieren Manager einer Organisation.
    <p>
    Seit: November 2004
    </p>

    <seealso cref="Person" />
    <seealso cref="Mitarbeiter" />
    <seealso cref="Organisation" />
    <seealso cref="OrganisationTesten" />

    Autor: Beringer Alfred
    </summary>
    */
    public class Manager : Mitarbeiter
    {
        // ---------------------------------------------------------------------
        // Klassenattribute
        // ---------------------------------------------------------------------

        /**
        <summary>
        Klasseninstanzvariable für Höchstgehalt
        (Klassenvariable aus der Klasse 'Mitarbeiter').

        <seealso cref="Mitarbeiter.MaxGehalt" />

        </summary>
        */
        private static float  MaxGehalt;

        /**
        <summary>
        Klassenvariable für Gewinnanteil.
        </summary>
        */
        private static float  Gewinnanteil = 0.05f;

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private Organisation geleiteteOrganisation;

        // ---------------------------------------------------------------------
        // Klassenkonstruktor
        // ---------------------------------------------------------------------

        /**
        <summary>
        Klassen-Initialisierungsblock.
        </summary>
        <remarks>
        Der Wert für 'MaxGehalt' könnte auch aus einer externen Datenquelle
        (z.B. aus einer Datenbank) kommen. <br />
        Ebenso könnte auch 'Gewinnanteil' hier initialisiert werden.
        </remarks>
        */
        static Manager()
        {
            SetMaxGehalt(15000.0f);
        }

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Konstruktor.

        <seealso cref="Person.Person(String, DateTime, char, String)" />

        </summary>
        <param name="name">
          Name
        </param>
        <param name="geburtsdatum">
          Geburtsdatum
        </param>
        <param name="geschlecht">
          Geschlecht
        </param>
        <param name="adresse">
          Adresse
        </param>
        */
        public Manager(String    name,
                       DateTime  geburtsdatum,
                       char      geschlecht,
                       String    adresse)
            : base(name, geburtsdatum, geschlecht, adresse)
        {
            Stellung = "Leiter";
            SetGrundgehalt(5000.0f);
        }

        /**
        <summary>
        Konstruktor.

        <seealso cref="Person.Person(String, DateTime, char)" />

        </summary>
        <param name="name">
          Name
        </param>
        <param name="geburtsdatum">
          Geburtsdatum
        </param>
        <param name="geschlecht">
          Geschlecht
        </param>
        */
        public Manager(String name, DateTime geburtsdatum, char geschlecht)
            : this(name, geburtsdatum, geschlecht, "")
        {
        }

        /**
        <summary>
        Konstruktor.

        <seealso cref="Person.Person(DateTime, char)" />

        </summary>
        <param name="geburtsdatum">
          Geburtsdatum
        </param>
        <param name="geschlecht">
          Geschlecht
        </param>
        */
        public Manager(DateTime geburtsdatum, char geschlecht)
            : this("", geburtsdatum, geschlecht, "")
        {
        }

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Property für die vom Manager geleitete Organisation.
        </summary>
        <remarks>
        Die Set-Methode ändert die vom Manager geleitete Organisation, ändert
        gegebenenfalls auch seinen Status in der Organisation.
        </remarks>
        */
        public Organisation GeleiteteOrganisation
        {
            get
            {
                return geleiteteOrganisation;
            }
            set
            {
                if (Orga != null)
                {
                    Orga.RemoveMitarbeiter(this);
                }
                Orga = value;
                this.geleiteteOrganisation = value;
                if (value != null)
                {
                    value.Leiter = this;
                }
            }
        }

        // ---------------------------------------------------------------------
        // Zugriffsmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Ändert das Grundgehalt des Managers (soferne es nicht größer 
        als das maximale Gehalt ist).
        </summary>
        <remarks>
        Diese Instanzmethode wird in jeder Subklasse identisch definiert
        (d.h. identisch überschrieben).
        </remarks>
        <param name="value">
          neues Grundgehalt - darf nicht größer als das maximale Gehalt sein
        </param>
        <returns>
          true, wenn Grundgehalt geändert wird
        </returns>
        */
        override public bool SetGrundgehalt(float value)
        {
            if (value <= Manager.MaxGehalt)
            {
                this.grundgehalt = value;
                return  true;
            }

            Console.WriteLine("Grundgehalt von " + this.GetType()
                              + " kann maximal sein: " + MaxGehalt);
            return  false;
        }

        // ---------------------------------------------------------------------
        // Sonstige Methoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert den Verdienst des Managers (= Grundgehalt + Prämie). <br />
        Überschreibt die Methode 'verdient' der Klasse Mitarbeiter.

        <seealso cref="Mitarbeiter.Verdient()" />

        </summary>
        <returns>
          Verdienst
        </returns>
        */
        override public float Verdient()
        {
            return  Grundgehalt
              + (geleiteteOrganisation.GewinnOhnePraemie() * GetGewinnanteil());
        }

        // ---------------------------------------------------------------------
        // Klassenmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert das Höchstgehalt eines Managers.

        <seealso cref="Mitarbeiter.getMaxGehalt()" />

        </summary>
        <remarks>
        Da die Klassen 'Manager' und 'Mitarbeiter' jeweils eigene 
        Klassenvariablen 'MaxGehalt' haben, müssen auch in jeder Klasse 
        eigene Zugriffsmethoden definiert werden.
        </remarks>
        <returns>
          Höchstgehalt
        </returns>
        */
        new public static float GetMaxGehalt()
        {
            return Manager.MaxGehalt;
        }

        /**
        <summary>
        Ändert die Höchstgrenze für das Grundgehalt eines Managers.
        </summary>
        <param name="value">
          neues Höchstgehalt
        </param>
        */
        new public static void SetMaxGehalt(float value)
        {
            Manager.MaxGehalt = value;
        }

        /**
        <summary>
        Liefert den Gewinnanteil eines Managers.
        </summary>
        <returns>
          Gewinnanteil
        </returns>
        */
        public static float GetGewinnanteil()
        {
            return  Gewinnanteil;
        }

        /**
        <summary>
        Ändert den Gewinnanteil eines Managers.
        </summary>
        <param name="value">
          neuer Gewinnanteil
        </param>
        */
        public static void SetGewinnanteil(float value)
        {
            Gewinnanteil = value;
        }
    }
}
