/*
 * Mitarbeiter.cs
 */
using System;
using System.Reflection;

namespace Beringer.oopBuch.Organisation
{
    /**
    <summary>
    Instanzen dieser Klasse repräsentieren Mitarbeiter einer Organisation.
    <p>
    Seit: November 2004
    </p>

    <seealso cref="Person" />
    <seealso cref="Organisation" />
    <seealso cref="OrganisationTesten" />

    Autor: Beringer Alfred
    </summary>
    */
    public class Mitarbeiter : Person
    {
        // ---------------------------------------------------------------------
        // Klassenattribute
        // ---------------------------------------------------------------------

        /**
        <summary>
        Klassenattribut für 'Mitarbeiter' und für alle 
        'Mitarbeiter'-Subklassen.
        </summary>
        */
        private static int  LetztePersNr = 0;

        /**
        <summary>
        Klassenattribut für Höchstgehalt, die für jede 
        'Mitarbeiter'-Subklasse unterschiedlich sein kann (entspricht somit 
        einer Klasseninstanzvariablen).

        <seealso cref="Manager.MaxGehalt" />

        </summary>
        */
        private static float  MaxGehalt;

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private int           persNr;
        private String        stellung = "Mitarbeiter";
        private Organisation  orga;

        protected float       grundgehalt;

        // ---------------------------------------------------------------------
        // Klassenkonstruktor
        // ---------------------------------------------------------------------

        /**
        <summary>
        Klassen-Initialisierungsblock.
        </summary>
        <remarks>
        Der Wert für 'MaxGehalt' könnte auch aus einer externen Datenquelle
        (z.B. aus einer Datenbank) kommen.
        </remarks>
        */
        static Mitarbeiter()
        {
            SetMaxGehalt(4500.0f);
        }

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Konstruktor.

        <seealso cref="Person.Person(String, DateTime, char, String)" />

        </summary>
        <param name="name">
          Name
        </param>
        <param name="geburtsdatum">
          Geburtsdatum
        </param>
        <param name="geschlecht">
          Geschlecht
        </param>
        <param name="adresse">
          Adresse
        </param>
        */
        public Mitarbeiter(String    name,
                           DateTime  geburtsdatum,
                           char      geschlecht,
                           String    adresse)
            : base(name, geburtsdatum, geschlecht, adresse)
        {
            persNrNeu();
            SetGrundgehalt(1200.0f);
        }

        /**
        <summary>
       Konstruktor.

        <seealso cref="Person.Person(String, DateTime, char)" />

        </summary>
        <param name="name">
          Name
        </param>
        <param name="geburtsdatum">
          Geburtsdatum
        </param>
        <param name="geschlecht">
          Geschlecht
        </param>
        */
        public Mitarbeiter(String name, DateTime geburtsdatum, char geschlecht)
            : this(name, geburtsdatum, geschlecht, "")
        {
        }

        /**
        <summary>
        Konstruktor.

        <seealso cref="Person.Person(DateTime, char)" />

        </summary>
        <param name="geburtsdatum">
          Geburtsdatum
        </param>
        <param name="geschlecht">
          Geschlecht
        </param>
        */
        public Mitarbeiter(DateTime geburtsdatum, char geschlecht)
            : this("", geburtsdatum, geschlecht, "")
        {
        }

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Get-Property für die (eindeutige) Personalnummer des Mitarbeiters.
        </summary>
        */
        public int PersNr
        {
            get
            {
                return persNr;
            }
        }

        /**
        <summary>
        Property für die Stellung (Position) des Mitarbeiters in der 
        Organisation.
        </summary>
        */
        public String Stellung
        {
            get
            {
                return stellung;
            }
            set
            {
                stellung = value;
            }
        }

        /**
        <summary>
        Get-Property für das Grundgehalt des Mitarbeiters.
        </summary>
        */
        public float Grundgehalt
        {
            get
            {
                return grundgehalt;
            }
        }

        /**
        <summary>
        Property für die Organisation des Mitarbeiters.
        </summary>
        <remarks>
        Die Set-Methode ändert die Organisation des Mitarbeiters und fügt den
        Mitarbeiter in die neue Organisation ein, falls er dieser noch nicht 
        angehört.
        </remarks>
        */
        public Organisation Orga
        {
            get
            {
                return orga;
            }
            set
            {
                if (this.orga != null)
                {
                    this.orga.RemoveMitarbeiter(this);
                }
                this.orga = value;
                if ( (this.orga != null) 
                                  && !this.orga.Mitarbeiterliste.Contains(this))
                {
                    this.orga.AddMitarbeiter(this);
                }
            }
        }

        // ---------------------------------------------------------------------
        // Zugriffsmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Ändert das Grundgehalt des Mitarbeiters (soferne es nicht größer 
        als das maximale Gehalt ist).
        </summary>
        <remarks>
        Diese Instanzmethode wird in jeder Subklasse identisch definiert
        (d.h. identisch überschrieben).
        </remarks>
        <param name="value">
          neues Grundgehalt - darf nicht größer als das maximale Gehalt sein
        </param>
        <returns>
          true, wenn Grundgehalt geändert wird
        </returns>
        */
        virtual public bool SetGrundgehalt(float value)
        {
            if (value <= Mitarbeiter.MaxGehalt)
            {
                this.grundgehalt = value;
                return  true;
            }

            Console.WriteLine("Grundgehalt von " + this.GetType()
                              + " kann maximal sein: " + MaxGehalt);
            return  false;
        }

        // ---------------------------------------------------------------------
        // Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert die Anrede des Mitarbeiters in der Organisation. <br />
        Überschreibt die Methode 'Anrede' der Klasse 'Person'.

         <seealso cref="Person.Anrede()"/>

        </summary>
        <returns>
          Anrede entsprechend dem Geschlecht und der Stellung des Mitarbeiters
        </returns>
        */
        override public String Anrede()
        {
            return  base.Anrede() + " " + Stellung + " " + Name;
        }

        /**
        <summary>
        Liefert den Verdienst des Mitarbeiters (entspricht bei einfachen
        Mitarbeitern dem Grundgehalt).
        </summary>
        <returns>
          Verdienst
        </returns>
        */
        virtual public float Verdient()
        {
            return  Grundgehalt;
        }

        // ---------------------------------------------------------------------
        // Nichtöffentliche Methoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Ordnet jedem Mitarbeiter eine eindeutige (fortlaufende) 
        Personalnummer zu.
        </summary>
        <returns>
          true, wenn Mitarbeiter noch keine Personalnummer hat
        </returns>
        */
        private bool persNrNeu()
        {
            if (persNr == 0)
            {
                setLetztePersNr(getLetztePersNr() + 1);
                persNr = getLetztePersNr();
                return  true;
            }

            // Mitarbeiter hat bereits eine Personalnummer
            return  false;
        }

        // ---------------------------------------------------------------------
        // Klassenmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert die derzeit letzte Personalnummer aller Mitarbeiter.
        </summary>
        <returns>
          letzte zugeordnete Personalnummer
        </returns>
        */
        protected static int getLetztePersNr()
        {
            return LetztePersNr;
        }

        /**
        <summary>
        Ändert die derzeit letzte Personalnummer aller Mitarbeiter.
        </summary>
        <param name="value">
          neue letzte zugeordnete Personalnummer
        </param>
        */
        private static void setLetztePersNr(int value)
        {
            Mitarbeiter.LetztePersNr = value;
        }

        /**
        <summary>
        Liefert das Höchstgehalt eines einfachen Mitarbeiters.
        </summary>
        <returns>
          Höchstgehalt
        </returns>
        */
        public static float GetMaxGehalt()
        {
            return  MaxGehalt;
        }

        /**
        <summary>
        Ändert die Höchstgrenze für das Grundgehalt eines einfachen 
        Mitarbeiters.
        </summary>
        <param name="value">
          neues Höchstgehalt
        </param>
        */
        public static void SetMaxGehalt(float value)
        {
            Mitarbeiter.MaxGehalt = value;
        }
    }
}
