/*
 * Organisation.cs
 */
using System;
using System.Collections;

namespace Beringer.oopBuch.Organisation
{
    /**
    <summary>
    Abstrakte Klasse für organisatorische Einheiten in einem 
    Unternehmen.<br />
    Solche organisatorischen Einheiten können Abteilungen, Projekte 
    usw. sein.
    <p>
    Seit: November 2004
    </p>

    <seealso cref="Mitarbeiter" />
    <seealso cref="Manager" />
    <seealso cref="Abteilung" />
    <seealso cref="Projekt" />
    <seealso cref="OrganisationTesten" />

    Autor: Beringer Alfred
    </summary>
    */
    public abstract class Organisation
    {
        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private String     name;
        private Manager    leiter;
        private ArrayList  mitarbeiterliste = new ArrayList();
        private float      sachkosten = 0.0f;

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Property für den Namen der Organisation.
        </summary>
        */
        public String Name
        {
            get
            {
                return name;
            }
            set
            {
                name = value;
            }
        }

        /**
        <summary>
        Property für den Leiter der Organisation.
        </summary>
        */
        public Manager Leiter
        {
            get
            {
                return leiter;
            }
            set
            {
                leiter = value;
            }
        }

        /**
        <summary>
        Property für die Sachkosten.
        </summary>
        */
        public float Sachkosten
        {
            get
            {
                return sachkosten;
            }
            set
            {
                sachkosten = value;
            }
        }

        /**
        <summary>
        Get-Property für die Mitarbeiterliste der Organisation.
        </summary>
        */
        public ArrayList Mitarbeiterliste
        {
            get
            {
                return mitarbeiterliste;
            }
        }

        // ---------------------------------------------------------------------
        // Zugriffsmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Fügt einen neuen Mitarbeiter hinzu; wird gegebenenfalls aus seiner
        vorhergehenden Organisation entfernt.
        </summary>
        <param name="einMitarbeiter">
          neuer (hinzuzufügender) Mitarbeiter
        </param>
        */
        public void AddMitarbeiter(Mitarbeiter einMitarbeiter)
        {
            Organisation  bisherigeOrg = einMitarbeiter.Orga;

            if ( (bisherigeOrg != null) && (bisherigeOrg != this) )
            {
                bisherigeOrg.RemoveMitarbeiter(einMitarbeiter);
                einMitarbeiter.Orga = this;
            }
            mitarbeiterliste.Add(einMitarbeiter);
        }

        /**
        <summary>
        Entfernt einen Mitarbeiter.
        </summary>
        <param name="einMitarbeiter">
          zu entfernender Mitarbeiter
        </param>
        */
        public void RemoveMitarbeiter(Mitarbeiter einMitarbeiter)
        {
            mitarbeiterliste.Remove(einMitarbeiter);
        }

        // ---------------------------------------------------------------------
        // Sonstige Methoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Erstellt eine Übersichtsliste aller Mitarbeiter + Leiter der 
        Organisation.
        </summary>
        */
        public void DruckeAlleMitarbeiter()
        {
            Console.WriteLine(this.GetType() + ": " + Name);
            if (leiter == null)
                Console.WriteLine("noch kein Manager als Leiter ernannt.");
            else
                Console.WriteLine("geleitet von: " + leiter.Name);

            Console.WriteLine("Anzahl Mitarbeiter = " + mitarbeiterliste.Count);
            foreach (Mitarbeiter  ma in mitarbeiterliste)
            {
                Console.WriteLine(ma.Anrede()
                                  + " (Grundgehalt = " + ma.Grundgehalt + ")");
            }
        }

        /**
        <summary>
        Erstellt eine Übersichtsliste der Gesamtkosten der Organisation.
        </summary>
        */
        public void DruckeKostenUebersicht()
        {
            Console.WriteLine(this.GetType() + ": " + Name + "   Kostenuebersicht");
            Console.WriteLine(
               "---------------------------------------------------------------------------");
            Console.WriteLine("Mitarbeiter \t\t\t Grundgehalt \t Verdienst");
            Console.WriteLine(
               "---------------------------------------------------------------------------");
            foreach (Mitarbeiter  ma in mitarbeiterliste)
            {
                Console.WriteLine(ma.Anrede() + " \t\t "
                                  + ma.Grundgehalt + " \t "
                                  + ma.Verdient());
            }
            Console.WriteLine(
               "---------------------------------------------------------------------------");
            Console.WriteLine("Personalkosten = \t " + Personalkosten());
            Console.WriteLine("Praemie =        \t " + Praemie());
            Console.WriteLine("Sachkosten =     \t " + Sachkosten);
            Console.WriteLine("Gesamtkosten =   \t " + Gesamtkosten());
            Console.WriteLine("Gewinn =         \t " + Gewinn());
        }

        /**
        <summary>
        Berechnet die Personalkosten aus den Grundgehältern der Mitarbeiter.
        </summary>
        <returns>
          Personalkosten ohne Prämie
        </returns>
        */
        public float PersonalkostenOhnePraemie()
        {
            float  summe = 0.0f;

            foreach (Mitarbeiter  ma in mitarbeiterliste)
            {
                summe = summe + ma.Grundgehalt;
            }
            return  summe;
        }

        /**
        <summary>
        Berechnet die Gesamtkosten (= Personalkosten + Sachkosten).
        </summary>
        <returns>
          Gesamtkosten ohne Prämie
        </returns>
        */
        public float GesamtkostenOhnePraemie()
        {
            return  PersonalkostenOhnePraemie() + Sachkosten;
        }

        /**
        <summary>
        Berechnet die Prämie des Leiters der Organisation.
        </summary>
        <returns>
          Prämie
        </returns>
        */
        public float Praemie()
        {
            return  GewinnOhnePraemie() * Manager.GetGewinnanteil();
        }

        /**
        <summary>
        Berechnet die Personalkosten aus den Grundgehältern + Prämien der 
        Mitarbeiter.
        </summary>
        <returns>
          Personalkosten + Prämien
        </returns>
        */
        public float Personalkosten()
        {
            return  PersonalkostenOhnePraemie() + Praemie();
        }

        /**
        <summary>
        Berechnet die Gesamtkosten (einschließlich der Prämien der 
        Mitarbeiter).
        </summary>
        <returns>
          Gesamtkosten
        </returns>
        */
        public float Gesamtkosten()
        {
            return  GesamtkostenOhnePraemie() + Praemie();
        }

        /**
        <summary>
        Berechnet den Gewinn der Organisation (ohne Prämien der Mitarbeiter).
        </summary>
        <returns>
          Gewinn
        </returns>
        */
        public float Gewinn()
        {
            return  GewinnOhnePraemie() - Praemie();
        }

        /**
        <summary>
        Berechnet den Gewinn der Organisation (zur Bestimmung des 
        Gewinnanteils). <br />
        Dieser ist abhängig von der Art der Organisation (Abteilung oder 
        Projekt).

        <seealso cref="Abteilung.GewinnOhnePraemie()"/>
        <seealso cref="Projekt.GewinnOhnePraemie()"/>

        </summary>
        <returns>
          Gewinn ohne Prämie
        </returns>
        */
        public abstract float GewinnOhnePraemie();

    }
}
