/*
 * Person.cs
 */
using System;
using System.Text;

namespace Beringer.oopBuch.Organisation
{
    /**
    <summary>
    Jede Instanz dieser Klasse repräsentiert eine natürliche Person.
    <p>
    Seit: November 2004
    </p>

    <seealso cref="Beringer.oopBuch.EinfacheBsp.Person1" />
    <seealso cref="System.DateTime" />
    <seealso cref="OrganisationMain" />

    Autor: Beringer Alfred
    </summary>
    <remark>
    Zusätzlich wurde die 'ToString'-Methode implementiert.
    </remark>
    */
    public class Person
    {
        // ---------------------------------------------------------------------
        // Klassenkonstante
        // ---------------------------------------------------------------------

        /**
        <summary>
        Kennzeichen für männliches Geschlecht. <br />
        Wird im Klassenkonstruktor initialisiert (z.B. mit 'M').
        </summary>
        */
        public static readonly char    MAENNLICH;

        /**
        <summary>
        Kennzeichen für weibliches Geschlecht. <br />
        Wird im Klassenkonstruktor initialisiert (z.B. mit 'W').
        </summary>
        */
        public static readonly char    WEIBLICH;

        /**
        <summary>
        Anrede für Personen männlichen Geschlechts. <br />
        Wird im Klassenkonstruktor initialisiert (z.B. mit "Herr").
        </summary>
        */
        public static readonly String  ANREDEMAENNLICH;

        /**
        <summary>
        Anrede für Personen weiblichen Geschlechts. <br />
        Wird im Klassenkonstruktor initialisiert (z.B. mit "Frau").
        </summary>
        */
        public static readonly String  ANREDEWEIBLICH;

        // ---------------------------------------------------------------------
        // Instanzattribute
        // ---------------------------------------------------------------------

        private String    name;
        private DateTime  geburtsdatum;
        private char      geschlecht;
        private String    adresse;

        // ---------------------------------------------------------------------
        // Klassenkonstruktor
        // ---------------------------------------------------------------------

        /**
        <summary>
        Klassenkonstruktor bzw. Klassen-Initialisierungsblock.
        </summary>
        <remarks>
        Die Werte für die nicht initialisierten Klassenkonstanten könnten von
        externen Datenquellen geholt werden, der Einfachheit halber werden 
        die Werte jedoch fix vorgegeben.
        <dl>
        <dt>Hinweis:
          <dd>Nur in dieser Methode sind Zuweisungen für Klassenkonstanten 
              erlaubt!
        </dl>
        </remarks>
        */
        static Person()
        {
            MAENNLICH = 'M';
            WEIBLICH  = 'W';
            ANREDEMAENNLICH = "Herr";
            ANREDEWEIBLICH  = "Frau";
        }

        // ---------------------------------------------------------------------
        // Konstruktoren
        // ---------------------------------------------------------------------

        /**
        <summary>
        Eine Person muss mindestens ein Geburtsdatum und ein Geschlecht 
        haben. Zusätzlich können der Name und die Adresse angegeben 
        werden. <br />
        Standardkonstruktor nicht erlaubt!

        <seealso cref="Person.Person(String, DateTime, char, String)" />

        </summary>
        <param name="name">
          Name
        </param>
        <param name="geburtsdatum">
          Geburtsdatum
        </param>
        <param name="geschlecht">
          Geschlecht
        </param>
        <param name="adresse">
          Adresse
        </param>
        */
        public Person(String    name,
                      DateTime  geburtsdatum,
                      char      geschlecht,
                      String    adresse)
        {
            Name = name;
            setGeburtsdatum(geburtsdatum);
            setGeschlecht(geschlecht);
            Adresse = adresse;
        }

        /**
        <summary>
        Eine Person muss mindestens ein Geburtsdatum und ein Geschlecht 
        haben. Zusätzlich kann der Name angegeben werden. <br />
        Die Adresse wird mit einer leeren Zeichenkette initialisiert. <br />
        Standardkonstruktor nicht erlaubt!

        <seealso cref="Person.Person(String, DateTime, char)" />

        </summary>
        <param name="name">
          Name
        </param>
        <param name="geburtsdatum">
          Geburtsdatum
        </param>
        <param name="geschlecht">
          Geschlecht
        </param>
        */
        public Person(String name, DateTime geburtsdatum, char geschlecht)
            : this(name, geburtsdatum, geschlecht, "")
        {
        }

        /**
        <summary>
        Eine Person muss mindestens ein Geburtsdatum und ein Geschlecht 
        haben. <br />
        Name und Adresse werden mit einer leeren Zeichenkette 
        initialisiert. <br />
        Standardkonstruktor nicht erlaubt!

        <seealso cref="Person.Person(DateTime, char)" />

        </summary>
        <param name="geburtsdatum">
          Geburtsdatum
        </param>
        <param name="geschlecht">
          Geschlecht
        </param>
        */
        public Person(DateTime geburtsdatum, char geschlecht)
            : this("", geburtsdatum, geschlecht, "")
        {
        }

        // ---------------------------------------------------------------------
        // Properties
        // ---------------------------------------------------------------------

        /**
        <summary>
        Property für den Namen der Person.
        </summary>
        */
        public String Name
        {
            get
            {
                return name;
            }
            set
            {
                name = value;
            }
        }

        /**
        <summary>
        Get-Property für das Geburtsdatum der Person.
        </summary>
        */
        public DateTime Geburtsdatum
        {
            get
            {
                /* DateTime ist eine Struktur (d.h. ein Wertetyp), daher kann 
                   das Originalobjekt zurückgegeben werden.
                */
                return geburtsdatum;
            }
        }

        /**
        <summary>
        Get-Property für das Geschlecht der Person.
        </summary>
        */
        public char Geschlecht
        {
            get
            {
                return geschlecht;
            }
        }

        /**
        <summary>
        Property für die Adresse der Person.
        </summary>
        */
        public String Adresse
        {
            get
            {
                return adresse;
            }
            set
            {
                adresse = value;
            }
        }

        // ---------------------------------------------------------------------
        // Öffentliche Instanzmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Liefert die Anrede für die Person.
        </summary>
        <returns>
          Anrede entsprechend dem Geschlecht bzw. leere Zeichenkette, falls
          das Geschlecht falsch ist
        </returns>
        */
        virtual public String Anrede()
        {
            if (geschlecht == MAENNLICH)
                return  ANREDEMAENNLICH;

            if (geschlecht == WEIBLICH)
                return  ANREDEWEIBLICH;

            return  "";
        }

        /**
        <summary>
        Berechnet das Alter der Person.
        </summary>
        <returns>
          Alter in Jahren
        </returns>
        */
        public int Alter()
        {
            DateTime  heute = DateTime.Today;
    
            return  heute.Year - geburtsdatum.Year;
        }

        // ---------------------------------------------------------------------
        // Methode 'ToString'
        // ---------------------------------------------------------------------

        /**
        <summary>
        Methode ToString (von Object) wird überschrieben.
        </summary>
        <remarks>
        String-Darstellung einer Personen-Instanz: <br />
               Person[ Name = ..., Geschlecht = ..., Geburtsdatum = tt. mm. jj,
                                                     Adresse = ... ]
        </remarks>
        <returns>
          String-Darstellung einer Personen-Instanz
        </returns>
        */
        override public String ToString()
        {
            StringBuilder  text  = new StringBuilder();
            DateTime       datum = Geburtsdatum;

            // Zugriff auf Attribute nur über Properties (Datenkapselung)!
            text.Append( GetType().FullName).Append(
                          "[ Name = ").Append(Name).Append(
                          ", Geschlecht = ").Append(Geschlecht).Append(
                          ", Geburtsdatum = ").Append(
                              datum.Day).Append(". ").Append(
                              datum.Month).Append(". ").Append(
                              datum.Year).Append(
                          ", Adresse = ").Append(Adresse).Append(
                          " ]" );
            return text.ToString();
        }

        // ---------------------------------------------------------------------
        // Nichtöffentliche Zugriffsmethoden
        // ---------------------------------------------------------------------

        /**
        <summary>
        Ändert das Geburtsdatum der Person.
        </summary>
        <remarks>
        Das neue Geburtsdatum muss vor dem aktuellen Tagesdatum liegen.
        </remarks>
        <param name="value">
          neues Geburtsdatum
        </param>
        <returns>
          true, wenn gültiges Geburtsdatum
        </returns>
        */
        private bool setGeburtsdatum(DateTime value)
        {
            DateTime  heute = DateTime.Today;

            if (value.CompareTo(heute) < 0)
            {
                /* DateTime ist eine Struktur (d.h. ein Wertetyp), daher kann 
                   das Parameterobjekt verwendet werden.
                */
                this.geburtsdatum = value;
                return  true;
            }

            // ungültiges Geburtsdatum
            this.geburtsdatum = heute;
            return  false;
        }

        /**
        <summary>
        Ändert das Geschlecht der Person.
        </summary>
        <param name="value">
          neues Geschlecht (männlich oder weiblich)
        </param>
        <returns>
          true, wenn gültiges Geschlecht
        </returns>
        */
        private bool setGeschlecht(char value)
        {
            this.geschlecht = value;
            return ((value == MAENNLICH) || (value == WEIBLICH)) ? true : false;
        }
    }
}
