/*
 * DAOerfassenSchueler.java
 */
package at.beringer.oopBuch.daoKlassen;

import java.io.*;

import at.beringer.oopBuch.modelKlassen.Schueler;

/**
DAO-Klasse für das Erfassen von Schülerdaten von der 
Standardeingabe (= Tastatur).
<p>
Die Schülerdaten werden in 'Schueler'-Objekten aufbereitet.
</p>
<p>
Methoden 'oeffnen' und 'schliessen' sind nicht notwendig, weil
'System.in' (= Standardeingabe Tastatur) standardmäßig immer geöffnet ist.
</p>
<p>
Bemerkungen:
<ul>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
  <li>'^Z' (= 'Strg' + 'Z') beendet die Eingabe (EOF) und liefert 'null' für die
      Eingabe zurück.</li>
  <li>Umgekehrt wird 'null' nur bei '^Z' als Eingabe zurückgegeben.</li>
  <li>EOF ist also äquivalent zur Eingabe von '^Z'.</li>
  <li>Achtung: 'EOFException' tritt bei der Standardeingabe nicht ein.</li>
  <li>Zusätzlich wird auch die Eingabe der Zeichenfolge "EOF" als EOF 
      interpretiert (gleichbedeutend mit der Eingabe von '^Z').</li>
  <li>Bei einem allgemeinen Eingabefehler ('IOException') wird 'null' als 
      Eingabe definiert (das ist gleichbedeutend mit EOF).</li>
</ul>
</p>

<hr />

@since August 2000
@version Juli 2002 (Zeichenkette mit variabler Länge)
@version <br />Juni 2007 (Refaktorierung)
@version <br />September 2012 (Refaktorierung - lesenEingabe)
@version <br />Juli 2014 (Verschieben in Paket 'daoKlassen')

@see Schueler

@see at.beringer.oopBuch.einAusgabe.ErfassenSchreiben
@see at.beringer.oopBuch.jdbc.ErfassenSchreibenDB

@see DAOlesenSchueler

@author Beringer Alfred
*/
public class DAOerfassenSchueler
{
    // -------------------------------------------------------------------------
    // Klassenkonstante
    // -------------------------------------------------------------------------

    /**
    Zeichenkette, die das Ende der Tastatureingabe definiert.
    <p>
    Dient als zusätzliche EOF-Kennzeichnung (zusätzlich zur Eingabe von '^Z')
    </p>
    */
    private static final String  EOF = "EOF";

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    /**
    Datenobjekt für die zu erfassenden Daten
    */
    private Schueler  schueler;
    private boolean   eof = false;

    /**
    Verbindungsobjekt zur Datei (= Tastatur)
    */
    private BufferedReader  tastatur = new BufferedReader(
                                              new InputStreamReader(System.in));

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert die Daten des erfassten Datensatzes.

    @return  Datenobjekt mit den Daten des erfassten Datensatzes
    */
    public Schueler getSchueler()
    {
        return  schueler;
    }

    // -------------------------------------------------------------------------

    /**
    Zeigt EOF (= "Ende der Datei") an (nachdem alle Datensätze erfasst wurden).

    @return  true, wenn das Ende der Datenerfassung erreicht wurde
    */
    public boolean getEof()
    {
        return  eof;
    }

    // -------------------------------------------------------------------------
    // Instanzmethoden für Dateiverarbeitung
    // -------------------------------------------------------------------------

    /**
    Lesen des nächsten Datensatzes.
    Prüfen und aufbereiten der Eingabedaten. Erstellen eines Schülerobjekts.
    <p>
    Die Datenfelder werden (entsprechend dem Satzaufbau) der Reihe nach über 
    die Tastatur eingegeben (erfasst) und - soweit möglich - geprüft.<br />
    Bei einem Fehler wird kein Schülerobjekt erstellt und die Erfassung des 
    Datensatzes abgebrochen.
    </p>

    @return  true, wenn die Erfassung des Datensatzes erfolgreich war
    */
    public boolean lesen()
    {
        String  eingabe;

        String  name;
        float   notenschnitt;
        String  klasse;

        // --------------------
        // 1. Datenfeld (Name - Zeichenkette mit variabler Länge)
        eingabe = lesenEingabe("Bitte Schülernamen eingeben (Ende: " 
                               + EOF + " oder ^Z):");
        if (eof)
        {
            return  true;     // EOF ist kein Fehler
        }

        name = eingabe;

        // --------------------
        // 2. Datenfeld (Notendurchschnitt - Zahl)
        eingabe = lesenEingabe("Bitte Notendurchschnitt eingeben:");
        if (eof)
        {
            // Bei zu frühem EOF wird kein Datensatz erstellt
            return  true;     // EOF ist kein Fehler
        }

        try
        {
            notenschnitt = Float.parseFloat(eingabe);
        }
        catch (NumberFormatException fehler)
        {
            System.out.println("Falscher Notendurchschnitt: " + eingabe);
            return  false;
        }

        // --------------------
        // 3. Datenfeld (Klassenbezeichnung - Zeichenkette mit fixer Länge)
        eingabe = lesenEingabe("Bitte Klasse eingeben (max. "
                               + Schueler.KLASSELEER.length() + " Zeichen):");
        if (eof)
        {
            // Bei zu frühem EOF wird kein Datensatz erstellt
            return  true;     // EOF ist kein Fehler
        }

        klasse = eingabe;

        // --------------------
        // Erfasste Daten sind korrekt -> Objekt für Datensatz instanzieren

        schueler = new Schueler();
        schueler.setName(name);
        schueler.setNotenschnitt(notenschnitt);
        schueler.setKlasse(klasse);

        return  true;
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Liest eine Zeile von der Tastatur.
    <p>
    Eingabe: alle Zeichen bis zur 'Return'- bzw. 'Enter'-Taste.
    </p>
    <p>
    Ende der Eingabe durch die Zeichenfolge "EOF" oder 
       durch '^Z' (= 'Strg' + 'Z').
    </p>
    Bemerkungen:
    <ul>
      <li>'EOFException' tritt bei der Standardeingabe nicht ein!</li>
      <li>'^Z' = EOF-Kennung bei Konsoleneingabe, d.h.:
          EOF ist äquivalent zur Eingabe von '^Z'.</li>
      <li>Eingabe von '^Z' liefert 'null' für die Eingabe.</li>
      <li>EOF ist also gleichbedeutend mit 'null' für die Eingabe.</li>
    </ul>

    @param  eingabeaufforderung  Eingabeaufforderung
     
    @return  eingegebene Daten oder 'null' (bei Eingabe von '^Z')
    */
    private String lesenEingabe(String eingabeaufforderung)
    {
        String eingabe;
        
        System.out.println(eingabeaufforderung);
        try
        {
            eingabe = tastatur.readLine();
        }
        catch (IOException fehler)
        {
            eingabe = null;               // Datenerfassung beenden (EOF setzen)
            System.out.println("Fehler beim Erfassen: " + fehler.getMessage());
        }

        // EOF prüfen:
        if ( (eingabe == null) || (eingabe.equals(EOF)) )
        {
            eof = true;
            System.out.println("EOF");     // diese Ausgabe dient nur zum Testen
        }

        return  eingabe;
    }
}
