/*
 * DAOlesenDBSchueler.java   
 */
package at.beringer.oopBuch.daoKlassen;

import java.util.*;
import java.sql.*;

import at.beringer.oopBuch.jdbc.DBconnection;
import at.beringer.oopBuch.modelKlassen.Schueler;

/**
DAO-Klasse für das Lesen von Schülerdaten aus einer Datenbank.
<p>
Das Lesen erfolgt mit Hilfe von SQL-SELECT-Befehlen. <br />
Das Ergebnis eines SELECT-Befehls ('ResultSet'-Daten) wird in eine generische 
'ArrayList<Schueler>'- bzw. 'ArrayList<String>'-Collection aufbereitet.
</p>
<p>
In dieser Klasse werden 3 SQL-SELECT-Befehle definiert:
<ul>
  <li>Schülerliste (alle Schüler).
      Die Schülerdaten werden in 'Schueler'-Objekten aufbereitet.</li>
  <li>Schülerliste (alle Schüler). 
      Die Schülerdaten werden in 'String'-Objekten aufbereitet.</li>
  <li>Klassenliste (alle Schüler einer Klasse). 
      Die Schülerdaten werden in 'Schueler'-Objekten aufbereitet.</li>
</ul>
</p>
<p>
Gezeigt wird die Verwendung von 'PreparedStatement'-Objekten.
</p>
<p>
Jedem SQL-Befehl wird ein eigenes 'String'-Objekt und ein eigenes
'PreparedStatement'-Objekt zugeordnet. <br />
Weiters entspricht jedem SQL-Befehl eine öffentliche Methode, die von der 
Applikation zur Durchführung des SQL-Befehls verwendet wird.
</p>
<p>
Bemerkungen:
<ul>
  <li>Die SQL-Befehle sind fix codiert.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
  <li>Die Konsolausgaben bei Fehlern dienen nur zur Kontrolle und sollten durch 
      Erzeugung eigener Fehlerobjekte ersetzt werden.</li>
  <li>Der Einfachheit halber sind die Fehlermeldungen nicht 
      internationalisiert!</li>
</ul>
</p>

<hr />

@since Juli 2012
@version Juli 2014 (Verschieben in Paket 'daoKlassen')

@see DBconnection
@see Schueler

@see at.beringer.oopBuch.jdbc.LesenDBAusgeben

@see DAOlesenSchueler

@author Beringer Alfred
*/
public class DAOlesenDBSchueler
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    /**
    Verbindungsobjekt zur Datenbank
    */
    private DBconnection  datenbank;

    // ----- SQL-Befehle mit entsprechenden 'PreparedStatement'-Objekten

    private String             sqlSelectAlleSchueler;
    private PreparedStatement  prepStatSelectAlleSchueler;
    private String             sqlSelectSchuelerKlasse;
    private PreparedStatement  prepStatSelectSchuelerKlasse;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Im Konstruktor werden die notwendigen Schritte gesetzt, um mit 
    der Datenbank (über SQL-Befehle) kommunizieren zu können.
    <p>
    Die benötigten SQL-Befehle werden definiert und die zugehörigen
    'PreparedStatement'-Objekte erzeugt.
    </p>
    */
    public DAOlesenDBSchueler()
    {
        datenbank = new DBconnection();
        definierenSQLBefehle();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Benötigte SQL-Befehle (mit Parameter) definieren. <br />
    Benötigte 'PreparedStatement'-Objekte erzeugen.
    <p>
    Die SQL-Befehle werden der Einfachheit halber fix codiert.
    </p> 
    Ein Parameter wird durch ein '?' im SQL-Befehls-String definiert.
    */
    private void definierenSQLBefehle()
    {
        sqlSelectAlleSchueler =
            "SELECT name, notenschnitt, klasse FROM schueler ";

        sqlSelectSchuelerKlasse = 
            "SELECT name, notenschnitt, klasse "
                + "FROM schueler "
                + "WHERE klasse = ? ";

        prepStatSelectAlleSchueler = 
            datenbank.erzeugenPrepStatement(sqlSelectAlleSchueler);

        prepStatSelectSchuelerKlasse = 
            datenbank.erzeugenPrepStatement(sqlSelectSchuelerKlasse);
    }

    // -------------------------------------------------------------------------
    // Instanzmethoden für den Zugriff auf die Datenbank (über SQL-SELECTs)
    // -------------------------------------------------------------------------

    /**
    Liefert Schülerdaten als Zeichenkette.

    @return  aufbereitetes Ergebnis des SQL-Befehls als List<String>-Objekt
    */
    public List<String> selectAlleSchuelerStr()
    {
        ResultSet     resultat;
        List<String>  ergebnis = new ArrayList<>();

        try
        {
            // SQL-Befehl durchführen
            resultat = prepStatSelectAlleSchueler.executeQuery();
            while (resultat.next())                   // alle Zeilen durchlaufen
            {
                // Achtung: Indexierung beginnt mit 1!

                ergebnis.add(resultat.getString(1)  + " " 
                             + resultat.getFloat(2) + " "
                             + resultat.getString(3));
            }
            resultat.close();
        }
        catch(SQLException fehler)
        {
            System.err.println("FehlerSQL.1: " + fehler.getMessage());
        }
        return  ergebnis;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert Schülerdaten als 'Schueler'-Objekt.

    @return  aufbereitetes Ergebnis des SQL-Befehls als List<Schueler>-Objekt
    */
    public List<Schueler> selectAlleSchueler()
    {
        ResultSet       resultat;
        List<Schueler>  ergebnis = new ArrayList<>();
        Schueler        schueler;

        try
        {
            // SQL-Befehl durchführen
            resultat = prepStatSelectAlleSchueler.executeQuery();
            while (resultat.next())                   // alle Zeilen durchlaufen
            {
                schueler = new Schueler();
                // Achtung: Indexierung beginnt mit 1!
                schueler.setName(resultat.getString(1));
                schueler.setNotenschnitt(resultat.getFloat(2));
                schueler.setKlasse(resultat.getString(3));

                ergebnis.add(schueler);
            }
            resultat.close();
        }
        catch(SQLException fehler)
        {
            System.err.println("FehlerSQL.2: " + fehler.getMessage());
        }
        return  ergebnis;
    }

    //--------------------------------------------------------------------------

    /**
    Liefert Schülerdaten als 'Schueler'-Objekt 
    (nur Schüler aus einer bestimmten Klasse).

    @param  klasse  Parameter für SQL-Befehl (Klasse)

    @return  aufbereitetes Ergebnis des SQL-Befehls als List<Schueler>-Objekt
    */
    public List<Schueler> selectSchuelerKlasse(String klasse)
    {
        ResultSet       resultat;
        List<Schueler>  ergebnis = new ArrayList<>();
        Schueler        schueler;

        try
        {
            // Parameter setzen
            prepStatSelectSchuelerKlasse.setString(1, klasse);
            // SQL-Befehl durchführen
            resultat = prepStatSelectSchuelerKlasse.executeQuery();
            while (resultat.next())                   // alle Zeilen durchlaufen
            {
                schueler = new Schueler();
                // Achtung: Indexierung beginnt mit 1!
                schueler.setName(resultat.getString(1));
                schueler.setNotenschnitt(resultat.getFloat(2));
                schueler.setKlasse(resultat.getString(3));

                ergebnis.add(schueler);
            }
            resultat.close();
        }
        catch(SQLException fehler)
        {
            System.err.println("FehlerSQL.3: " + fehler.getMessage());
        }
        return  ergebnis;
    }

    // -------------------------------------------------------------------------

    /**
    Schließt die Verbindung zur Datenbank.
    <p>
    Diese Methode ist öffentlich, da sie von der Applikation im Allg. am Ende 
    aufgerufen wird.
    </p>
    */
    public void schliessenVerbindung()
    {
        datenbank.schliessenVerbindung();
    }
}
