/*
 * DAOlesenSchueler.java
 */
package at.beringer.oopBuch.daoKlassen;

import java.io.*;

import at.beringer.oopBuch.modelKlassen.Schueler;

/**
DAO-Klasse für das Lesen von Schülerdaten aus einer Datei 
(Nichtstandarddatei).
<p>
Die Schülerdaten werden in 'Schueler'-Objekten aufbereitet.
</p>
<p>
Bemerkungen:
<ul>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
  <li>Die Konsolausgaben bei Fehlern dienen nur zur Kontrolle und sollten durch 
      Erzeugung eigener Fehlerobjekte ersetzt werden.</li>
</ul>
</p>
<p>
Satzspiegel der Schülerdatei:
<pre>
---------------------------------------------------------------
 Stellen
von - bis (Länge)   Datenfeld
---------------------------------------------------------------
  1 -  4   (4) .... LÄNGE DES NACHFOLGENDEN NAMENS (int)
  5 - xx  (var) ... NAME (variable Länge)
           (4) .... NOTENDURCHSCHNITT (float)
           (7) .... KLASSE (fixe Länge)
</pre>
</p>

<hr />

@since August 2000
@version Juli 2002 (Zeichenkette mit variabler Länge)
@version <br />Juni 2007 (Refaktorierung)
@version <br />Juli 2012 (FileInputStream-Objekt als lokale Variable)
@version <br />Juli 2014 (Verschieben in Paket 'daoKlassen')

@see Schueler

@see at.beringer.oopBuch.einAusgabe.LesenAusgeben
@see at.beringer.oopBuch.einAusgabeSwing.LesenAusgebenStringV

@see DAOlesenDBSchueler
@see DAOerfassenSchueler

@author Beringer Alfred
*/
public class DAOlesenSchueler
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    /**
    Datenobjekt für die zu lesenden Daten
    */
    private Schueler  schueler;
    private boolean   eof = false;

    /**
    Verbindungsobjekt zur Datei
    */
    private DataInputStream  eingabesatz;

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert die Daten des gelesenen Datensatzes.

    @return  Datenobjekt mit den Daten des gelesenen Datensatzes
    */
    public Schueler getSchueler()
    {
        return  schueler;
    }

    // -------------------------------------------------------------------------

    /**
    Zeigt EOF (= "Ende der Datei") an (nachdem alle Datensätze gelesen wurden).

    @return  true, wenn das Ende der Datei erreicht wurde
    */
    public boolean getEof()
    {
        return  eof;
    }

    // -------------------------------------------------------------------------
    // Instanzmethoden für Dateiverarbeitung
    // -------------------------------------------------------------------------

    /**
    Öffnet eine (Nichtstandard-)Datei zum Lesen.

    @param  dateiname  Name der zu öffnenden Datei (als Zeichenkette)

    @return  true, wenn das Öffnen der Datei erfolgreich war
    */
    public boolean oeffnen(String dateiname)
    {
        FileInputStream  eingabedatei;              // entfällt bei Schachtelung

        try
        {
            eingabedatei = new FileInputStream(dateiname);
            eingabesatz  = new DataInputStream(eingabedatei);

// Schachtelung ist möglich:
//          eingabesatz = new DataInputStream(new FileInputStream(dateiname));
        }
        catch (IOException fehler)
        {
            System.err.println("Fehler beim Öffnen der Datei: " 
                               + dateiname + " " 
                               + fehler.getMessage());         // nur zum Testen
            return  false;
        }

        return  true;
    }

    // -------------------------------------------------------------------------

    /**
    Lesen des nächsten Datensatzes aus der Datei - alle Attribute eines 
    Schülerobjekts sind nebeneinander ohne Abstand gespeichert.
    Aufbereiten der Eingabedaten. Erstellen eines Schülerobjekts.
    <p>
    Die Datenfelder werden (entsprechend dem Satzaufbau) der Reihe nach aus der 
    Datei gelesen.<br />
    Bei einem IO-Fehler wird kein Schülerobjekt erstellt.
    </p>
    <p>
    Hinweis: Die Daten werden nicht mehr geprüft.
    </p>
    <p>
    Satzaufbau für die Daten eines Schülers:
    <ol>
      <li>Länge (= Anzahl der Zeichen) des nachfolgenden Namens (int-Zahl)</li>
      <li>Name (Zeichenkette mit variabler Länge)</li>
      <li>Notendurchschnitt (float-Zahl)</li>
      <li>Klasse (Zeichenkette mit fixer Länge)</li>
    </ol>
    </p>

    @return  true, wenn das Lesen des Datensatzes erfolgreich war
    */
    public boolean lesen()
    {
        String  name;
        float   notenschnitt;
        String  klasse;

        // byte-Array für eine Zeichenkette mit fixer Länge
        byte[]  bytesKlasse = new byte[Schueler.KLASSELEER.length()];
        // byte-Array für eine Zeichenkette mit variabler Länge
        byte[]  bytesVar;

        try
        {
            // Länge des 1. Datenfeldes
            bytesVar = new byte[eingabesatz.readInt()];

            // 1. Datenfeld (Name - Zeichenkette mit variabler Länge)
            eingabesatz.read(bytesVar);
            name = new String(bytesVar);

            // 2. Datenfeld (Notendurchschnitt - Zahl)
            notenschnitt = eingabesatz.readFloat();

            // 3. Datenfeld (Klassenbezeichnung - Zeichenkette mit fixer Länge)
            eingabesatz.read(bytesKlasse);
            klasse = new String(bytesKlasse);
        }
        catch (EOFException exc)
        {
            eof = true;
            System.out.println("EOF");     // diese Ausgabe dient nur zum Testen
            return  true;      // EOF ist kein Fehler
        }
        catch (IOException fehler)
        {
            System.err.println("Fehler beim Lesen des Datensatzes: " 
                               + fehler.getMessage());         // nur zum Testen
            return  false;
        }

        // --------------------
        // Daten wurden korrekt gelesen -> Objekt für Datensatz instanzieren

        schueler = new Schueler();
        schueler.setName(name);
        schueler.setNotenschnitt(notenschnitt);
        schueler.setKlasse(klasse);

        return  true;
    }

    // -------------------------------------------------------------------------

    /**
    Schließt die Datei.

    @return  true, wenn das Schließen der Datei erfolgreich war
    */
    public boolean schliessen()
    {
        try
        {
            eingabesatz.close();
        }
        catch (Exception fehler)
        {
            System.err.println("Fehler beim Schließen der Datei: " 
                               + fehler.getMessage());         // nur zum Testen
            return  false;
        }

        return  true;
    }
}
