/*
 * DAOschreibenDBSchueler.java   
 */
package at.beringer.oopBuch.daoKlassen;

import java.sql.*;

import at.beringer.oopBuch.jdbc.DBconnection;
import at.beringer.oopBuch.modelKlassen.Schueler;

/**
DAO-Klasse für das Schreiben von Schülerdaten in eine Datenbank.
<p>
Die Schülerdaten sind die Attribute eines 'Schueler'-Objekts.
</p>
<p>
Das Schreiben erfolgt mit Hilfe von SQL-INSERT-Befehlen.
</p>
<p>
Gezeigt wird die Verwendung von 'PreparedStatement'-Objekten.
</p>
<p>
Jedem SQL-Befehl wird ein eigenes 'String'-Objekt und ein eigenes
'PreparedStatement'-Objekt zugeordnet. <br />
Weiters entspricht jedem SQL-Befehl eine öffentliche Methode, die von der 
Applikation zur Durchführung des SQL-Befehls verwendet wird.
</p>
<p>
Bemerkungen:
<ul>
  <li>Die SQL-Befehle sind fix codiert.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
  <li>Die Konsolausgaben bei Fehlern dienen nur zur Kontrolle und sollten durch 
      Erzeugung eigener Fehlerobjekte ersetzt werden.</li>
  <li>Der Einfachheit halber sind die Fehlermeldungen nicht 
      internationalisiert!</li>
</ul>
</p>

<hr />

@since Juli 2012
@version Juli 2014 (Verschieben in Paket 'daoKlassen')

@see DBconnection
@see Schueler

@see at.beringer.oopBuch.jdbc.ErfassenSchreibenDB

@see DAOschreibenSchueler

@author Beringer Alfred
*/
public class DAOschreibenDBSchueler
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    /**
    Datenobjekt für die zu schreibenden Daten
    */
    private Schueler  schueler;

    /**
    Verbindungsobjekt zur Datenbank
    */
    private DBconnection  datenbank;

    // ----- SQL-Befehle mit entsprechenden 'PreparedStatement'-Objekten

    private String             sqlInsertSchueler;
    private PreparedStatement  prepStatInsertSchueler;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Im Konstruktor werden die notwendigen Schritte gesetzt, um mit 
    der Datenbank (über SQL-Befehle) kommunizieren zu können.
    <p>
    Die benötigten SQL-Befehle werden definiert und die zugehörigen
    'PreparedStatement'-Objekte erzeugt.
    </p>
    */
    public DAOschreibenDBSchueler()
    {
        datenbank = new DBconnection();
        definierenSQLBefehle();
    }

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Setzt die Daten für den zu schreibenden Datensatz.

    @param  schueler  Datenobjekt mit den Daten des zu schreibenden Datensatzes
    */
    public void setSchueler(Schueler schueler)
    {
        this.schueler = schueler;
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Benötigte SQL-Befehle (mit Parameter) definieren. <br />
    Benötigte 'PreparedStatement'-Objekte erzeugen.
    <p>
    Die SQL-Befehle werden der Einfachheit halber fix codiert.
    </p> 
    Ein Parameter wird durch ein '?' im SQL-Befehls-String definiert.
    */
    private void definierenSQLBefehle()
    {
        sqlInsertSchueler =
            "INSERT INTO schueler "
                         + "(name, notenschnitt, klasse) "
                         + "VALUES (?, ?, ?) ";

        prepStatInsertSchueler = 
                datenbank.erzeugenPrepStatement(sqlInsertSchueler);
    }

    // -------------------------------------------------------------------------
    // Instanzmethoden für den Zugriff auf die Datenbank (über SQL-INSERT)
    // -------------------------------------------------------------------------

    /**
    Schreiben eines neuen Datensatzes in die Datenbank 

    @return  true, wenn das Schreiben des Datensatzes erfolgreich war
    */
    public boolean schreiben()
    {
        try
        {
            /*
            Parameter setzen.

            Bei mehreren Parametern ist die Reihenfolge der Argumente 
            wesentlich! 
            Die Reihenfolge entspricht genau der Reihenfolge der ?-Parameter im
            entsprechenden SQL-Befehl.

            Achtung: Indexierung beginnt mit 1!
            */
            prepStatInsertSchueler.setString(1, schueler.getName());
            prepStatInsertSchueler.setFloat(2,  schueler.getNotenschnitt());
            prepStatInsertSchueler.setString(3, schueler.getKlasse());

            // SQL-Befehl durchführen
            prepStatInsertSchueler.execute();
        }
        catch(SQLException fehler)
        {
            System.err.println("FehlerSQL.3: " + fehler.getMessage());
            return  false;
        }

        return  true;
    }

    // -------------------------------------------------------------------------

    /**
    Schließt die Verbindung zur Datenbank.
    <p>
    Diese Methode ist öffentlich, da sie von der Applikation im Allg. am Ende 
    aufgerufen wird.
    </p>
    */
    public void schliessenVerbindung()
    {
        datenbank.schliessenVerbindung();
    }
}
