/*
 * DAOschreibenSchueler.java
 */
package at.beringer.oopBuch.daoKlassen;

import java.io.*;

import at.beringer.oopBuch.modelKlassen.Schueler;

/**
DAO-Klasse für das Schreiben von Schülerdaten in eine Datei 
(Nichtstandarddatei).
<p>
Die Schülerdaten sind die Attribute eines 'Schueler'-Objekts.
</p>
<p>
Bemerkungen:
<ul>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
  <li>Die Konsolausgaben bei Fehlern dienen nur zur Kontrolle und sollten 
      durch Erzeugung eigener Fehlerobjekte ersetzt werden.</li>
</ul>
</p>
<p>
Satzspiegel der Schülerdatei:
<pre>
---------------------------------------------------------------
 Stellen
von - bis (Länge)   Datenfeld
---------------------------------------------------------------
  1 -  4   (4) .... LÄNGE DES NACHFOLGENDEN NAMENS (int)
  5 - xx  (var) ... NAME (variable Länge)
           (4) .... NOTENDURCHSCHNITT (float)
           (7) .... KLASSE (fixe Länge)
</pre>
</p>

<hr />

@since August 2000
@version Juli 2002 (Zeichenkette mit variabler Länge)
@version <br />Juni 2007 (Refaktorierung)
@version <br />Juli 2012 (FileOutputStream-Objekt als lokale Variable)
@version <br />Juli 2014 (Verschieben in Paket 'daoKlassen')

@see Schueler

@see at.beringer.oopBuch.einAusgabe.ErfassenSchreiben
@see at.beringer.oopBuch.einAusgabeSwing.ErfassenSchreibenV

@see DAOschreibenDBSchueler
@see DAOausgebenSchueler

@author Beringer Alfred
*/
public class DAOschreibenSchueler
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    /**
    Datenobjekt für die zu schreibenden Daten
    */
    private Schueler  schueler;

    /**
    Verbindungsobjekt zur Datei
    */
    private DataOutputStream  ausgabesatz;

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Setzt die Daten für den zu schreibenden Datensatz.

    @param  schueler  Datenobjekt mit den Daten des zu schreibenden Datensatzes
    */
    public void setSchueler(Schueler schueler)
    {
        this.schueler = schueler;
    }

    // -------------------------------------------------------------------------
    // Instanzmethoden für Dateiverarbeitung
    // -------------------------------------------------------------------------

    /**
    Öffnet eine (Nichtstandard-)Datei zum Schreiben.

    @param  dateiname  Name der zu öffnenden Datei (als Zeichenkette)

    @return  true, wenn das Öffnen der Datei erfolgreich war
    */
    public boolean oeffnen(String dateiname)
    {
        FileOutputStream  ausgabedatei;             // entfällt bei Schachtelung

        try
        {
            ausgabedatei = new FileOutputStream(dateiname);
            ausgabesatz  = new DataOutputStream(ausgabedatei);

// Schachtelung ist möglich:
//          ausgabesatz = new DataOutputStream(new FileOutputStream(dateiname));
        }
        catch (IOException fehler)
        {
            System.err.println("Fehler beim Öffnen der Datei: " 
                               + dateiname + " " 
                               + fehler.getMessage());         // nur zum Testen
            return  false;
        }

        return  true;
    }

    // -------------------------------------------------------------------------

    /**
    Schreiben eines neuen Datensatzes in die Datei.
    <p>
    Die Datenfelder werden (entsprechend dem Satzaufbau) der Reihe nach in die 
    Datei geschrieben.
    </p>
    <p>
    Satzaufbau für die Daten eines Schülers:
    <ol>
      <li>Länge (= Anzahl der Zeichen) des nachfolgenden Namens (int-Zahl)</li>
      <li>Name (Zeichenkette mit variabler Länge)</li>
      <li>Notendurchschnitt (float-Zahl)</li>
      <li>Klasse (Zeichenkette mit fixer Länge)</li>
    </ol>
    </p>

    @return  true, wenn das Schreiben des Datensatzes erfolgreich war
    */
    public boolean schreiben()
    {
        try
        {
            // Speichern der Länge des 1. Datenfeldes
            ausgabesatz.writeInt(schueler.getName().length());

            // 1. Datenfeld (Name - Zeichenkette mit variabler Länge)
            ausgabesatz.writeBytes(schueler.getName());
                  /*
                  Bem.:   'writeBytes' schreibt ASCII-Zeichen (8 Bit)
                          'writeChars' schreibt Unicode-Zeichen (16 Bit)
                  */

            // 2. Datenfeld (Notendurchschnitt - Zahl)
            ausgabesatz.writeFloat(schueler.getNotenschnitt());

            // 3. Datenfeld (Klasse - Zeichenkette mit fixer Länge)
            ausgabesatz.writeBytes(schueler.getKlasse());
        }
        catch (IOException fehler)
        {
            System.err.println("Fehler beim Schreiben des Datensatzes: "
                               + fehler.getMessage());         // nur zum Testen
            return  false;
        }

        return  true;
    }

    // -------------------------------------------------------------------------

    /**
    Schließt die Datei.

    @return  true, wenn das Schließen der Datei erfolgreich war
    */
    public boolean schliessen()
    {
        try
        {
            ausgabesatz.close();
        }
        catch (Exception fehler)
        {
            System.err.println("Fehler beim Schließen der Datei: " 
                               + fehler.getMessage());         // nur zum Testen
            return  false;
        }

        return  true;
    }
}
