/*
 * Datum.java
 */
package at.beringer.oopBuch.einfacheBsp;

import java.time.LocalDate;

/**
Eine einfache Datumsklasse.
<p>
Instanzen dieser Klasse können kopiert werden.
</p>

<hr />

@since August 1999
@version April 2002 (überschreiben 'Object.clone()')
@version <br />April 2014 (aktualisieren Java 8 - 'GregorianCalendar' ersetzen)

@see DatumTesten
@see Datum7

@author Beringer Alfred
*/
public class Datum implements Cloneable
{
    // -------------------------------------------------------------------------
    // Klassenkonstante
    // -------------------------------------------------------------------------

    private static final String[]  Monatsnamen  = { "Jänner   ",
                                                    "Februar  ",
                                                    "März     ",
                                                    "April    ",
                                                    "Mai      ",
                                                    "Juni     ",
                                                    "Juli     ",
                                                    "August   ",
                                                    "September",
                                                    "Oktober  ",
                                                    "November ",
                                                    "Dezember "  };

    private static final int[]     TageProMonat
                = { 31,  28,  31,  30,  31,  30,  31,  31,  30,  31,  30,  31 };

    private static final int[]     LfdTagMonatsanfang
                = {  1,  32,  60,  91, 121, 152, 182, 213, 244, 274, 305, 335 };

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private int  jj;
    private int  mm;
    private int  tt;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Standardkonstruktor.
    <p>
    Initialisierung mit aktuellem Tagesdatum
    </p>
    */
    public Datum()
    {
        LocalDate  heute = LocalDate.now();      // liefert aktuelles Tagesdatum

        jj = heute.getYear();
        mm = heute.getMonthValue();
        tt = heute.getDayOfMonth();
    }

    // -------------------------------------------------------------------------

    /**
    Initialisierung mit einem beliebigen Tagesdatum.
    <p>
    Dieses Datum wird auf Gültigkeit geprüft, bei einem Fehler wird das
    aktuelle Tagesdatum zur Initialisierung verwendet.
    </p>

    @param  jahr   Jahr
    @param  monat  Monatsnummer (bei 1 (= Jänner) beginnend)
    @param  tag    Tag
    */
    public Datum(int jahr, int monat, int tag)
    {
        this();                       // initialisieren mit aktuellem Tagesdatum
        setDatum(jahr, monat, tag);
    }

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert das Jahr.

    @return  Jahr
    */
    public int jj()
    {
        return  jj;
    }

    // ----------

    /**
    Liefert die Monatsnummer.

    @return  Monatsnummer
    */
    public int mm()
    {
        return  mm;
    }

    // ----------

    /**
    Liefert den Tag.

    @return  Tag
    */
    public int tt()
    {
        return  tt;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert den Monatsnamen entsprechend der Monatsnummer.

    @return  Monatsname
    */
    public String monatsname()
    {
        return  Monatsnamen[mm-1];
    }

    // -------------------------------------------------------------------------
    // Private Set-Methoden
    // -------------------------------------------------------------------------

    /**
    Prüft und setzt ein korrektes Datum.
    <p>
    Der Einfachheit halber werden Fehlermeldungen auf die Konsole geschrieben.
    </p>

    @param  jahr   Jahr
    @param  monat  Monatsnummer
    @param  tag    Tag

    @return  true, wenn das Datum korrekt ist
    */
    private boolean setDatum(int jahr, int monat, int tag)
    {
        int  tageProMonat;

        if (jahr == 0)
        {
            System.err.println("Fehler: " + jahr + " ist ungültig");
            return false;
        }

        if ((monat < 1) || (monat > 12))
        {
            System.err.println("Fehler: " + monat + " ist ungültig");
            return false;
        }
            
        tageProMonat = TageProMonat[monat-1];
        if ((monat == 2) && (Datum.istSchaltjahr(jahr)))
        {
            tageProMonat++;
        }
        if ((tag < 1) || (tag > tageProMonat))
        {
            System.err.println("Fehler: " + tag + " ist ungültig");
            return false;
        }

        jj = jahr;
        mm = monat;
        tt = tag;

        return  true;
    }

    // -------------------------------------------------------------------------
    // Sonstige Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Liefert den laufenden Tag des Jahres.

    @return  laufender Tag des Jahres
    */
    public int lfdTagDesJahres()
    {
        int  lfd;

        lfd = LfdTagMonatsanfang[mm-1] + tt - 1;
        if ( (mm > 2) && (this.istSchaltjahr()) )
        {
            lfd++;
        }
        return  lfd;
    }

    // -------------------------------------------------------------------------

    /**
    Prüft das eigene Jahr auf Schaltjahr.

    @return  true, wenn das eigene Jahr ein Schaltjahr ist
    */
    public boolean istSchaltjahr()
    {
        return  Datum.istSchaltjahr(jj);
    }

    // -------------------------------------------------------------------------
    // Methoden 'toString' und 'clone'
    // -------------------------------------------------------------------------

    /**
    Überschreiben der Methode 'String Object.toString()'.
    <p>
    Liefert eine beschreibende Text-Darstellung einer Instanz der Klasse:
    <pre>
      TT. MM. JJJJ
    </pre>
    </p>

    @see java.lang.Object#toString()

    @return  Datum in der Form "TT. MM. JJJJ"
    */
    @Override
    public String toString()
    {
        return  tt + ". " + mm + ". " + jj;
    }

    // -------------------------------------------------------------------------

    /**
    Überschreiben der Methode 'Object Object.clone()'.
    <p>
    Kopieren einer Datum-Instanz (tiefe Kopie):
    <ol>
      <li>Sichtbarkeit auf 'public' anheben ('Object.clone()' ist 
          'protected'!).</li>
      <li>Ausnahme 'CloneNotSupportedException' abfangen
          (auch wenn die Klasse 'Datum' die Kennzeichenschnittstelle 'Cloneable'
           bereits implementiert!).</li>
      <li>Vererbte Methode 'Object.clone()' aufrufen ('super.clone()').</li>
      <li>Veränderliche Attribute kopieren: <br />
          Da diese Klasse keine veränderlichen Attribute hat, ist für eine tiefe
          Kopie nichts weiter zu tun. <br />
          (Wenn bereits in Get-/Set-Methoden nur Kopien von veränderlichen 
           Attributen verwendet werden, könnten diese Methoden für das Kopieren 
           dieser Attribute verwendet werden.)</li>
    </ol>
    </p>

    @see java.lang.Object#clone()

    @return  (tiefe) Kopie eines Datums
    */
    @Override
    public Object clone()
    {
        Datum  datumKopie;

        try
        {
            datumKopie = (Datum) super.clone();
      
            /* hier folgt das Kopieren der veränderlichen Attribute
               (für eine tiefe Kopie)
               z.B.:   kopie.setAttribut(getAttribut());
             */

            return  datumKopie;
        }
        catch (CloneNotSupportedException e)
        {
            // zurückgeben eines Dummy-Wertes!
            return  null;
        }
    }

    // -------------------------------------------------------------------------
    // Klassenmethoden
    // -------------------------------------------------------------------------

    /**
    Liefert das aktuelle Tagesdatum.

    @return  Instanz des aktuellen Tagesdatums
    */
    public static Datum heute()
    {
        return  new Datum();
    }

    // -------------------------------------------------------------------------

    /**
    Prüft ein vorgegebenes Jahr auf Schaltjahr.

    @param  jahr  zu prüfende Jahreszahl

    @return  true, wenn 'jahr' ein Schaltjahr ist
    */
    public static boolean istSchaltjahr(int jahr)
    {
        return  ((jahr%4 != 0) || ((jahr%100 == 0) && (jahr%400 != 0))) ? false
                                                                        : true;
    }
}
