/*
 * DBconnection.java   
 */
package at.beringer.oopBuch.jdbc;

import java.sql.DriverManager;
import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.SQLException;

import static at.beringer.oopBuch.jdbc.DBconnectionKonstante.*;

/**
Klasse für die Herstellung einer Verbindung zu einer DB und zur Erzeugung von 
'PreparedStatement'-Objekten.
<p>
Objekte dieser Klasse sind Attribute in einer DAO-Klasse <br />
(DAO ... Data Access Object).
</p>
<p>
Bemerkungen:
<ul>
  <li>Die Klassenkonstante 'CONNECTIONSTRING' für die URL der DB-Verbindung wird
      von einer Properties-Datei übernommen.</li>
  <li>Die Klassenkonstante 'DBMS' dient nur zum Testen bei den Konsolausgaben.
  <li>Die URL der DB-Verbindung beinhaltet nicht nur den DB-Server sondern auch 
      bereits den Namen der DB. <br />
      Man könnte den Namen der DB ebenso über eine weitere Properties-Datei 
      parametrisieren - dies wird aber der Einfachheit halber nicht 
      gemacht.</li>
  <li>Der Einfachheit halber ist die Fehlerbearbeitung sehr primitiv!</li>
  <li>Die Konsolausgaben bei Fehlern dienen nur zur Kontrolle und sollten durch 
      Erzeugung eigener Fehlerobjekte ersetzt werden.</li>
  <li>Der Einfachheit halber sind die Fehlermeldungen nicht 
      internationalisiert!</li>
  <li>Die sonstigen Konsolausgaben dienen nur zum Testen.
  <li>Ab Java Version 6.0 (bzw. JDBC 4.0) geschieht das Laden der Treiberklasse 
      automatisch.</li>
</ul>
</p>

<hr />

@since Juli 2012

@see DBconnectionKonstante
@see at.beringer.oopBuch.daoKlassen.DAOschreibenDBSchueler
@see at.beringer.oopBuch.daoKlassen.DAOlesenDBSchueler

@author Beringer Alfred
*/
public class DBconnection
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    /**
    Verbindungsobjekt zur Datenbank
    */
    private Connection  verbindung;

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Im Konstruktor werden die notwendigen Schritte für die Verbindung zur
    Datenbank durchgeführt (DB öffnen).
    <p>
    Der 'DriverManager' sucht automatisch unter den geladenen Treibern den 
    "richtigen" Treiber und stellt eine offene Verbindung zu der durch die 
    angegebene URL festgelegten DB her.
    </p>
    <p>
    Bem.: Der "richtige" Treiber ist jener, zu dem die angegebene URL "passt", 
          d.h. jener, der die angegebene URL kennt.
    </p>
    */
    public DBconnection()
    {
        try
        {
            verbindung = DriverManager.getConnection(CONNECTIONSTRING);
            System.out.println("Verbindung erfolgreich aufgebaut zu DB "
                               + DBMS);
        }
        catch(SQLException fehler)
        {
            fehler.printStackTrace();
            System.err.println("Fehler1-conn: " + fehler.getMessage()); 
            System.exit(0);
        }
    }

    // -------------------------------------------------------------------------
    // Öffentliche Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    'PreparedStatement'-Objekt erzeugen für einen SQL-Befehl.
    <p>
    Die SQL-Befehle selbst und die Parameter zu den SQL-Befehlen werden 
    in der DAO-Klasse definiert. <br />
    Diese Methode wird nur in der DAO-Klasse benötigt und sollte dort
    pro SQL-Befehl nur 1 einziges Mal durchgeführt werden!
    </p>

    @param  sqlBefehl  SQL-Befehl für 'PreparedStatement'-Objekt

    @return  das erzeugte 'PreparedStatement'-Objekt
    */
    public PreparedStatement erzeugenPrepStatement(String sqlBefehl)
    {
        PreparedStatement  prepStatement = null;

        try
        {
            prepStatement = verbindung.prepareStatement(sqlBefehl);
            System.out.println("Prepared-Statement erfolgreich erzeugt");
        }
        catch(SQLException fehler)
        {
            System.err.println("Fehler2-stat: " + fehler.getMessage());
        }
        return  prepStatement;
    }

    //--------------------------------------------------------------------------

    /**
    Schließt die Verbindung zur DB.
    <p>
    Zu dieser Methode muss es in der DAO-Klasse eine entsprechende 
    öffentliche Methode geben, die von der Applikation aufgerufen wird.
    </p>
    */
    public void schliessenVerbindung()
    {
        try      
        {
            verbindung.close();
            System.out.println("Verbindung erfolgreich aufgelöst");
        }
        catch(Exception fehler)
        {
            System.err.println("Fehler3-close: " + fehler.getMessage());
        }
    }
}
