/*
 * Mitarbeiter.java
 */
package at.beringer.oopBuch.organisation;

import java.time.LocalDate;
import java.lang.reflect.*;

/**
Instanzen dieser Klasse repräsentieren Mitarbeiter einer Organisation.

<hr />

@since August 1999
@version April 2014 (aktualisieren Java 8 - 'GregorianCalendar' ersetzen)

@see Person
@see Organisation
@see OrganisationTesten

@author Beringer Alfred
*/
public class Mitarbeiter extends Person
{
    // -------------------------------------------------------------------------
    // Klassenattribute
    // -------------------------------------------------------------------------

    /**
    Klassenattribut für 'Mitarbeiter' und für alle 'Mitarbeiter'-Subklassen.
    */
    private static int  LetztePersNr = 0;

    /**
    Klassenattribut für Höchstgehalt, die für jede 'Mitarbeiter'-Subklasse
    unterschiedlich sein kann (entspricht somit einem Klasseninstanzattribut).

    @see Manager#MaxGehalt
    */
    private static float  MaxGehalt;

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private int           persNr;
    private String        stellung = "Mitarbeiter";
    private float         grundgehalt;
    private Organisation  organisation;

    // -------------------------------------------------------------------------
    // Klassenkonstruktor
    // -------------------------------------------------------------------------

    /**
    Klassenkonstruktor (Klassen-Initialisierungsblock).
    <p>
    Die Werte für nicht initialisierte Klassenattribute könnten von externen 
    Datenquellen geholt werden (z.B. von einer Datei, einer Datenbank oder von 
    einer Properties-Datei).
    <p>
    Der Einfachheit halber werden die Werte jedoch fix vorgegeben.
    </p>
    */
    static
    {
      setMaxGehalt(4500.0f);
    }

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    @see Person#Person(String, LocalDate, char, String)

    @param  name          Name
    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    @param  adresse       Adresse
    */
    public Mitarbeiter(String     name,
                       LocalDate  geburtsdatum,
                       char       geschlecht,
                       String     adresse)
    {
        super(name, geburtsdatum, geschlecht, adresse);
        persNrNeu();
        setGrundgehalt(1200.0f);
    }

    // ----------

    /**
    @see Person#Person(String, LocalDate, char)

    @param  name          Name
    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    */
    public Mitarbeiter(String name, LocalDate geburtsdatum, char geschlecht)
    {
        this(name, geburtsdatum, geschlecht, "");
    }

    // ----------

    /**
    @see Person#Person(LocalDate, char)

    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    */
    public Mitarbeiter(LocalDate geburtsdatum, char geschlecht)
    {
        this("", geburtsdatum, geschlecht, "");
    }

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert die (eindeutige) Personalnummer des Mitarbeiters.

    @return  Personalnummer
    */
    public int getPersNr()
    {
        return  persNr;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert die Stellung (Position) des Mitarbeiters in der Organisation.

    @return  Stellung
    */
    public String getStellung()
    {
        return  stellung;
    }

    // ----------

    /**
    Ändert die Stellung (Position) des Mitarbeiters in der Organisation.

    @param  stellung  neue Stellung
    */
    public void setStellung(String stellung)
    {
       this.stellung = stellung;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert das Grundgehalt des Mitarbeiters.

    @return  Grundgehalt
    */
    public float getGrundgehalt()
    {
        return  grundgehalt;
    }

    // ----------

    /**
    Ändert das Grundgehalt des Mitarbeiters (soferne es nicht größer als
    das maximale Gehalt ist).
    <p>
    Mit Hilfe des Reflection-APIs wird die korrekte 
    'getMaxGehalt'-Klassenmethode aufgerufen (ohne Reflection-API würde immer 
    die 'getMaxGehalt'-Methode der Klasse 'Mitarbeiter' aufgerufen 
    werden!). <br />
    Ohne Reflection-API müsste diese Instanzmethode in jeder Subklasse 
    identisch definiert (d.h. identisch überschrieben) werden.
    </p>

    @param  grundgehalt  neues Grundgehalt - darf nicht größer als das maximale
                         Gehalt sein
 
    @return  true, wenn Grundgehalt geändert wird
    */
    public boolean setGrundgehalt(float grundgehalt)
    {
        Class   c = this.getClass();
        Method  m;
        Float   rueck;
        float   maxGehalt;

        try
        {
            // gesuchte Methode hat keine Parameter, daher Parameter-Array leer
            m = c.getMethod("getMaxGehalt", new Class[0]);

            /* aufgerufene Methode ist eine Klassenmethode, daher 'null'
              aufgerufene Methode hat keine Parameter, daher Argument-Array leer
              Rückgabetyp ist immer 'Object', daher Konvertierung auf 'Float' */
            rueck = (Float) m.invoke(null, new Object[0]);

            // Konvertierung in elementaren Datentyp
            maxGehalt = rueck.floatValue();
        }
        catch (Exception e)
        {
            maxGehalt = getMaxGehalt();   // dies ist immer die 
                                          // 'Mitarbeiter'-Methode!
        }

        if (grundgehalt <= maxGehalt)
        {
            this.grundgehalt = grundgehalt;
            return  true;
        }

        System.out.println("Grundgehalt von " + this.getClass()
                                              + " kann maximal sein: " 
                                              + maxGehalt);
        return  false;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert die Organisation des Mitarbeiters.

    @return  Organisation
    */
    public Organisation getOrganisation()
    {
        return  organisation;
    }

    // ----------

    /**
    Ändert die Organisation des Mitarbeiters und fügt Mitarbeiter in die neue
    Organisation ein, falls er dieser noch nicht angehört.

    @param  organisation  neue Organisation
    */
    public void setOrganisation(Organisation organisation)
    {
        if ( this.organisation != null )
        {
            this.organisation.removeMitarbeiter(this);
        }
        this.organisation = organisation;
        if ( (this.organisation != null) &&
                       !this.organisation.getMitarbeiterliste().contains(this) )
        {
            this.organisation.addMitarbeiter(this);
        }
    }

    // -------------------------------------------------------------------------
    // Sonstige Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Liefert die Anrede des Mitarbeiters in der Organisation. <br />
    Überschreibt die Methode 'anrede' der Klasse 'Person'.

    @see Person#anrede()

    @return  Anrede entsprechend dem Geschlecht und der Stellung des 
             Mitarbeiters
    */
    @Override
    public String anrede()
    {
        return  super.anrede() + " " + getStellung() + " " + getName();
    }

    // -------------------------------------------------------------------------

    /**
    Liefert den Verdienst des Mitarbeiters (entspricht bei einfachen
    Mitarbeitern dem Grundgehalt).

    @return  Verdienst
    */
    public float verdient()
    {
        return  getGrundgehalt();
    }

    // -------------------------------------------------------------------------
    // Private Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Ordnet jedem Mitarbeiter eine eindeutige (fortlaufende) Personalnummer zu.

    @return  true, wenn Mitarbeiter noch keine Personalnummer hat
    */
    private boolean persNrNeu()
    {
        if (persNr == 0)
        {
            setLetztePersNr(getLetztePersNr() + 1);
            persNr = getLetztePersNr();
            return  true;
        }

        // Mitarbeiter hat bereits eine Personalnummer
        return  false;
    }

    // -------------------------------------------------------------------------
    // Klassenmethoden
    // -------------------------------------------------------------------------

    /**
    Liefert die derzeit letzte Personalnummer aller Mitarbeiter.

    @return  letzte zugeordnete Personalnummer
    */
    protected static int getLetztePersNr()
    {
        return  LetztePersNr;
    }

    // ----------

    /**
    Ändert die derzeit letzte Personalnummer aller Mitarbeiter.

    @param  letztePersNr  neue letzte zugeordnete Personalnummer
    */
    private static void setLetztePersNr(int letztePersNr)
    {
        LetztePersNr = letztePersNr;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert das Höchstgehalt eines einfachen Mitarbeiters.

    @return  Höchstgehalt
    */
    public static float getMaxGehalt()
    {
        return  MaxGehalt;
    }

    // ----------

    /**
    Ändert die Höchstgrenze für das Grundgehalt eines einfachen Mitarbeiters.

    @param  maxGehalt  neues Höchstgehalt
    */
    public static void setMaxGehalt(float maxGehalt)
    {
        MaxGehalt = maxGehalt;
    }
}
