/*
 * Organisation.java
 */
package at.beringer.oopBuch.organisation;

import java.util.*;

/**
Abstrakte Klasse für organisatorische Einheiten in einem Unternehmen. <br />
Solche organisatorischen Einheiten können Abteilungen, Projekte usw. sein.

<hr />

@since August 1999
@version Jänner 2002 (Set)
@version <br />August 2004 (Fehler bei 'leiter == null' behoben,
                            Zugriffsmethoden für 'mitarbeiterliste' nicht mehr 
                           'public')
@version <br />Dezember 2004 (JDK 5.0: Generics für 'mitarbeiterliste',
                                       Verwendung von 'for-each')
@version <br />Juli 2014 (JDK 8.0: Datenstrom ('stream') + 
                        Aggregat-Operationen 'forEach', 'mapToDouble' und 'sum')

@see Mitarbeiter
@see Manager
@see Abteilung
@see Projekt
@see OrganisationTesten

@author Beringer Alfred
*/
public abstract class Organisation
{
    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private String            name;
    private Manager           leiter;
    private Set<Mitarbeiter>  mitarbeiterliste = new HashSet<>();
    private float             sachkosten  = 0.0f;

    // -------------------------------------------------------------------------
    // Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert den Namen der Organisation.

    @return  Name
    */
    public String getName()
    {
        return  name;
    }

    // ----------

    /**
    Ändert den Namen der Organisation.

    @param  name  neuer Name
    */
    public void setName(String name)
    {
        this.name = name;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert den Leiter der Organisation.

    @return  Leiter der Organisation
    */
    public Manager getLeiter()
    {
        return  leiter;
    }

    // ----------

    /**
    Ändert den Leiter der Organisation.

    @param  leiter  neuer Leiter der Organisation
    */
    public void setLeiter(Manager leiter)
    {
        this.leiter = leiter;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert die Sachkosten.

    @return  Sachkosten
    */
    public float getSachkosten()
    {
        return  sachkosten;
    }

    // ----------

    /**
    Ändert die Sachkosten.

    @param  sachkosten  neue Sachkosten
    */
    public void setSachkosten(float sachkosten)
    {
        this.sachkosten = sachkosten;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert die Mitarbeiterliste der Organisation.
    <p>
    Eine öffentliche Get-Methode sollte ein unveränderbares 'Collection'-Objekt
    retournieren ('Collections.unmodifiableSet(...)' - siehe JDK-Dokumentation).
    </p>

    @return  Mitarbeiterliste
    */
    protected Set<Mitarbeiter> getMitarbeiterliste()
    {
        return  this.mitarbeiterliste;
    }

    // ----------

    /**
    Fügt einen neuen Mitarbeiter hinzu; wird gegebenenfalls aus seiner
    vorhergehenden Organisation entfernt.

    @param  einMitarbeiter  neuer (hinzuzufügender) Mitarbeiter
    */
    public void addMitarbeiter(Mitarbeiter einMitarbeiter)
    {
        Organisation  bisherigeOrg = einMitarbeiter.getOrganisation();

        if ( (bisherigeOrg != null) && (bisherigeOrg != this) )
        {
            bisherigeOrg.removeMitarbeiter(einMitarbeiter);
            einMitarbeiter.setOrganisation(this);
        }
        mitarbeiterliste.add(einMitarbeiter);
    }

    // ----------

    /**
    Entfernt einen Mitarbeiter.

    @param  einMitarbeiter  zu entfernender Mitarbeiter
    */
    public void removeMitarbeiter(Mitarbeiter einMitarbeiter)
    {
        mitarbeiterliste.remove(einMitarbeiter);
    }

    // -------------------------------------------------------------------------
    // Sonstige Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Erstellt eine Übersichtsliste aller Mitarbeiter + Leiter der Organisation.
    */
    public void druckeAlleMitarbeiter()
    {
        System.out.println(this.getClass() + ": " + getName());
        if (leiter == null || leiter.getClass() != Manager.class)
        {
            System.out.println("noch kein Manager als Leiter ernannt.");
        }
        else
        {
            System.out.println("geleitet von: " + leiter.getName());
        }

        System.out.println("Anzahl Mitarbeiter = " + mitarbeiterliste.size());

        // Datenstrom ('stream') + Aggregat-Operation 'forEach' (seit JDK 8.0):
        mitarbeiterliste.stream()
                        .forEach(ma -> 
                             System.out.println(ma.anrede()
                                                + " (Grundgehalt = " 
                                                + ma.getGrundgehalt() + ")"));

        /* ---------- for-each-Schleife (seit JDK 5.0):
        for (Mitarbeiter ma : mitarbeiterliste)
        {
            System.out.println(ma.anrede()
                               + " (Grundgehalt = " 
                               + ma.getGrundgehalt() + ")");
        }
        ---------- */

        /* ---------- mit Iterator:
        Iterator  iterMA = mitarbeiterliste.iterator();
        while (iterMA.hasNext())
        {
            Mitarbeiter ma = (Mitarbeiter) iterMA.next();
            System.out.println(ma.anrede()
                               + " (Grundgehalt = " 
                               + ma.getGrundgehalt() + ")");
        }
        ---------- */
    }

    // -------------------------------------------------------------------------

    /**
    Erstellt eine Übersichtsliste der Gesamtkosten der Organisation.
    */
    public void druckeKostenUebersicht()
    {
        System.out.println(this.getClass() + ": " + getName() 
                           + "   Kostenuebersicht");
        System.out.println(
           "---------------------------------------------------------------------------");
        System.out.println("Mitarbeiter \t\t\t Grundgehalt \t Verdienst");
        System.out.println(
           "---------------------------------------------------------------------------");

        // Datenstrom ('stream') + Aggregat-Operation 'forEach' (seit JDK 8.0):
        mitarbeiterliste.stream()
                        .forEach(ma -> 
                             System.out.println(ma.anrede() + " \t\t "
                                                + ma.getGrundgehalt() + " \t "
                                                + ma.verdient()));
                        
        /* ---------- for-each-Schleife (seit JDK 5.0):
        for (Mitarbeiter ma : mitarbeiterliste)
        {
            System.out.println(ma.anrede() + " \t\t "
                               + ma.getGrundgehalt() + " \t "
                               + ma.verdient());
        }
        ---------- */

        /* ---------- mit Iterator:
        Iterator  iterMA = mitarbeiterliste.iterator();
        while (iterMA.hasNext())
        {
            Mitarbeiter ma = (Mitarbeiter) iterMA.next();
            System.out.println(ma.anrede() + " \t\t "
                               + ma.getGrundgehalt() + " \t "
                               + ma.verdient());
        }
        ---------- */

        System.out.println(
           "---------------------------------------------------------------------------");
        System.out.println("Personalkosten = \t " + personalkosten());
        System.out.println("Praemie =        \t " + praemie());
        System.out.println("Sachkosten =     \t " + getSachkosten());
        System.out.println("Gesamtkosten =   \t " + gesamtkosten());
        System.out.println("Gewinn =         \t " + gewinn());
    }

    // -------------------------------------------------------------------------

    /**
    Berechnet die Personalkosten aus den Grundgehältern der Mitarbeiter.

    @return  Personalkosten ohne Prämie
    */
    public float personalkostenOhnePraemie()
    {
        float  summe = 0.0f;

        // Datenstrom ('stream') + Aggregat-Operationen 
        //                         'mapToDouble' und 'sum' (seit JDK 8.0):
        summe = (float) mitarbeiterliste.stream()
                                        .mapToDouble(ma -> ma.getGrundgehalt())
                                        .sum();

        /* --- oder mit Methodenreferenz:
//      summe = (float)mitarbeiterliste.stream()
//                                     .mapToDouble(Mitarbeiter::getGrundgehalt)
//                                     .sum();

        /* ---------- for-each-Schleife (seit JDK 5.0):
        for (Mitarbeiter ma : mitarbeiterliste)
        {
            summe = summe + ma.getGrundgehalt();
        }
        ---------- */

        /* ---------- mit Iterator:
        Iterator  iterMA = mitarbeiterliste.iterator();
        while (iterMA.hasNext())
        {
          Mitarbeiter ma = (Mitarbeiter) iterMA.next();
          summe = summe + ma.getGrundgehalt();
        }
        ---------- */

        return  summe;
    }

    // -------------------------------------------------------------------------

    /**
    Berechnet die Gesamtkosten (= Personalkosten + Sachkosten).

    @return  Gesamtkosten ohne Prämie
    */
    public float gesamtkostenOhnePraemie()
    {
        return  personalkostenOhnePraemie() + getSachkosten();
    }

    // -------------------------------------------------------------------------

    /**
    Berechnet die Prämie des Leiters der Organisation.

    @return  Prämie
    */
    public float praemie()
    {
        return  gewinnOhnePraemie() * Manager.getGewinnanteil();
    }

    // -------------------------------------------------------------------------

    /**
    Berechnet die Personalkosten aus den Grundgehältern + Prämien der 
    Mitarbeiter.

    @return  Personalkosten + Prämien
    */
    public float personalkosten()
    {
        return  personalkostenOhnePraemie() + praemie();
    }

    // -------------------------------------------------------------------------

    /**
    Berechnet die Gesamtkosten (einschließlich der Prämien der Mitarbeiter).

    @return  Gesamtkosten
    */
    public float gesamtkosten()
    {
        return  gesamtkostenOhnePraemie() + praemie();
    }

    // -------------------------------------------------------------------------

    /**
    Berechnet den Gewinn der Organisation (ohne Prämien der Mitarbeiter).

    @return  Gewinn
    */
    public float gewinn()
    {
        return  gewinnOhnePraemie() - praemie();
    }

    // -------------------------------------------------------------------------

    /**
    Berechnet den Gewinn der Organisation (zur Bestimmung des 
    Gewinnanteils). <br />
    Dieser ist abhängig von der Art der Organisation (Abteilung oder Projekt).

    @see Abteilung#gewinnOhnePraemie()
    @see Projekt#gewinnOhnePraemie()

    @return  Gewinn ohne Prämie
    */
    public abstract float gewinnOhnePraemie();
}
