/*
 * Person.java
 */
package at.beringer.oopBuch.organisation;

import java.time.LocalDate;

/**
Jede Instanz dieser Klasse repräsentiert eine natürliche Person.
<p>
Zusätzlich wurde die 'toString'-Methode implementiert.
</p>

<hr />

@since August 1999
@version April 2002 (überschreiben 'toString')
@version <br />Juli 2003 (öffentliche Klassenkonstante mit 
                          Initialisierungsmethode)
@version <br />April 2014 (aktualisieren Java 8 - 'GregorianCalendar' ersetzen)

@see OrganisationTesten
@see Kunde
@see Mitarbeiter
@see at.beringer.oopBuch.einfacheBsp.Person

@author Beringer Alfred
*/
public class Person
{
    // -------------------------------------------------------------------------
    // Klassenkonstante
    // -------------------------------------------------------------------------

    /**
    Kennzeichen für das Geschlecht "männlich"
    */
    public static final char    MAENNLICH;

    /**
    Kennzeichen für das Geschlecht "weiblich"
    */
    public static final char    WEIBLICH;

    /**
    Anrede für Personen männlichen Geschlechts
    */
    public static final String  ANREDEMAENNLICH;

    /**
    Anrede für Personen weiblichen Geschlechts
    */
    public static final String  ANREDEWEIBLICH;

    // -------------------------------------------------------------------------
    // Instanzattribute
    // -------------------------------------------------------------------------

    private String     name;
    private LocalDate  geburtsdatum;
    private char       geschlecht;
    private String     adresse;
    
    // -------------------------------------------------------------------------
    // Klassenkonstruktor
    // -------------------------------------------------------------------------

    /**
    Klassenkonstruktor (Klassen-Initialisierungsblock).
    <p>
    Die Werte für nicht initialisierte Klassenkonstante könnten von externen 
    Datenquellen geholt werden (z.B. von einer Datei, einer Datenbank oder von 
    einer Properties-Datei).
    <p>
    Der Einfachheit halber werden die Werte jedoch fix vorgegeben.
    </p>
    <dl>
      <dt>Hinweis:</dt>
        <dd>Nur in dieser Methode sind Zuweisungen für Klassenkonstante 
            erlaubt!</dd>
    </dl>
    */
    static
    {
        MAENNLICH = 'M';
        WEIBLICH  = 'W';
        ANREDEMAENNLICH = "Herr";
        ANREDEWEIBLICH  = "Frau";
    }

    // -------------------------------------------------------------------------
    // Konstruktoren
    // -------------------------------------------------------------------------

    /**
    Eine Person muss mindestens ein Geburtsdatum und ein Geschlecht haben.
    Zusätzlich können der Name und die Adresse angegeben werden. <br />
    Standardkonstruktor nicht erlaubt!

    @param  name          Name
    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    @param  adresse       Adresse
    */
    public Person(String     name,
                  LocalDate  geburtsdatum,
                  char       geschlecht,
                  String     adresse)
    {
        setName(name);
        setGeburtsdatum(geburtsdatum);
        setGeschlecht(geschlecht);
        setAdresse(adresse);
    }

    // ----------

    /**
    Eine Person muss mindestens ein Geburtsdatum und ein Geschlecht haben.
    Zusätzlich kann der Name angegeben werden. <br />
    Die Adresse wird mit einer leeren Zeichenkette initialisiert. <br />
    Standardkonstruktor nicht erlaubt!

    @param  name          Name
    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    */
    public Person(String name, LocalDate geburtsdatum, char geschlecht)
    {
        this(name, geburtsdatum, geschlecht, "");
    }

    // ----------

    /**
    Eine Person muss mindestens ein Geburtsdatum und ein Geschlecht 
    haben. <br />
    Name und Adresse werden mit einer leeren Zeichenkette initialisiert. <br />
    Standardkonstruktor nicht erlaubt!

    @param  geburtsdatum  Geburtsdatum
    @param  geschlecht    Geschlecht
    */
    public Person(LocalDate geburtsdatum, char geschlecht)
    {
        this("", geburtsdatum, geschlecht, "");
    }

    // -------------------------------------------------------------------------
    // Öffentliche Zugriffsmethoden (Get- und Set-Methoden)
    // -------------------------------------------------------------------------

    /**
    Liefert den Namen der Person.

    @return  Name
    */
    public String getName()
    {
        return  name;
    }

    // ----------

    /**
    Ändert den Namen der Person.

    @param  name  neuer Name
    */
    public void setName(String name)
    {
        this.name = name;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert das Geburtsdatum der Person.
    Ein LocalDate-Objekt ist immutable (wie ein String-Objekt) und kann daher 
    wie ein elementarer Datentyp verwendet werden.
    Insbesondere kann es direkt als Parameter in Methoden verwendet werden (es 
    ist nicht notwendig, eine Kopie als Parameter zu uebergeben).

    @return  Geburtsdatums
    */
    public LocalDate getGeburtsdatum()
    {
        // Originalobjekt kann zurückgegeben werden
        return  geburtsdatum;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert das Geschlecht der Person.

    @return  Geschlecht
    */
    public char getGeschlecht()
    {
        return  geschlecht;
    }

    // -------------------------------------------------------------------------

    /**
    Liefert die Adresse der Person.

    @return  Adresse
    */
    public String getAdresse()
    {
        return  adresse;
    }

    // ----------

    /**
    Ändert die Adresse der Person.

    @param  adresse  neue Adresse
    */
    public void setAdresse(String adresse)
    {
        this.adresse = adresse;
    }

    // -------------------------------------------------------------------------
    // Sonstige öffentliche Instanzmethoden
    // -------------------------------------------------------------------------

    /**
    Liefert die Anrede für die Person.

    @return  Anrede entsprechend dem Geschlecht bzw. leere Zeichenkette, falls
                                                     das Geschlecht falsch ist
    */
    public String anrede()
    {
        if (geschlecht == MAENNLICH)
        {
            return ANREDEMAENNLICH;
        }

        if (geschlecht == WEIBLICH)
        {
            return ANREDEWEIBLICH;
        }

        return "";
    }

    // -------------------------------------------------------------------------

    /**
    Berechnet das Alter der Person.

    @return  Alter in Jahren
    */
    public int alterBerechnen()
    {
        LocalDate  heute = LocalDate.now();
      
        return  heute.getYear() - geburtsdatum.getYear();
    }

    // -------------------------------------------------------------------------
    // Methode 'toString'
    // -------------------------------------------------------------------------

    /**
    Überschreiben der Methode 'String Object.toString()'.
    <p>
    Liefert eine beschreibende Text-Darstellung einer Instanz der Klasse:
    <pre>
         Person [ Name = ..., 
                  Geschlecht = ..., 
                  Geburtsdatum = tt. mm. jj,
                  Adresse = ... ]
    </pre>
    </p>

    @see java.lang.Object#toString()

    @return  Zeichenkette, die eine Instanz beschreibt
    */
    @Override
    public String toString()
    {
        StringBuilder  text  = new StringBuilder();
        LocalDate      datum = getGeburtsdatum();

        // Zugriff auf Attribute nur über Zugriffsmethoden (Datenkapselung)!
        text.append(getClass().getName()).append(
                     " [ Name = "       ).append(getName()      ).append(
                     ", Geschlecht = "  ).append(getGeschlecht()).append(
                     ", Geburtsdatum = ").append(
                          datum.getDayOfMonth()).append(". ").append(
                          datum.getMonthValue()).append(". ").append(
                          datum.getYear()      ).append(
                     ", Adresse = "     ).append(getAdresse()   ).append(
                     " ]" );

        return  new String(text);
    }

    // -------------------------------------------------------------------------
    // Private Zugriffsmethoden
    // -------------------------------------------------------------------------

    /**
    Ändert das Geburtsdatum der Person.
    <p>
    Das neue Geburtsdatum muss vor dem aktuellen Tagesdatum liegen.
    </p>

    @param  geburtsdatum  neues Geburtsdatum

    @return  true, wenn 'geburtsdatum' ein gültiges Geburtsdatum ist
    */
    private boolean setGeburtsdatum(LocalDate geburtsdatum)
    {
        LocalDate  heute = LocalDate.now();

        if (geburtsdatum.isBefore(heute))
        {
            // Parameterobjekt kann direkt verwendet werden
            this.geburtsdatum = geburtsdatum;
            return  true;
        }

        // ungültiges Geburtsdatum
        this.geburtsdatum = heute;
        return  false;
    }

    // -------------------------------------------------------------------------

    /**
    Ändert das Geschlecht der Person.
    <p>
    Prüft, ob das Geschlecht gültig ist (Kennzeichen muss ein gültiges 
    Geschlechtskennzeichen - männlich bzw. weiblich - sein).
    </p>

    @param  geschlecht  neues Geschlecht (männlich oder weiblich)

    @return  true, wenn 'geschlecht' ein gültiges Geschlechtskennzeichen ist
    */
    private boolean setGeschlecht(char geschlecht)
    {
        this.geschlecht = geschlecht;
        return  ((geschlecht == MAENNLICH) || (geschlecht == WEIBLICH)) ? true
                                                                        : false;
    }
}
